import os
import unittest
from unittest import mock

from parameterized import parameterized_class

from checkov.common.images.image_referencer import Image
from checkov.terraform.image_referencer.provider.gcp import GcpTerraformProvider
from tests.graph_utils.utils import set_graph_by_graph_framework, PARAMETERIZED_GRAPH_FRAMEWORKS, \
    add_vertices_to_graph_by_graph_framework


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestGcp(unittest.TestCase):
    def setUp(self) -> None:
        self.graph = set_graph_by_graph_framework(self.graph_framework)

    def test_extract_images_from_resources(self):
        # given
        resource = {
            "file_path_": "/cloud_run.tf",
            "__end_line__": 17,
            "__start_line__": 1,
            "template": {
                "spec": {
                    "containers": {
                        "image": "gcr.io/cloudrun/hello",
                    }
                }
            },
            "resource_type": "google_cloud_run_service",
        }
        self.graph = set_graph_by_graph_framework(self.graph_framework)
        add_vertices_to_graph_by_graph_framework(self.graph_framework, resource, self.graph)

        # when
        with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': self.graph_framework}):
            gcp_provider = GcpTerraformProvider(graph_connector=self.graph)
            images = gcp_provider.extract_images_from_resources()

        # then
        assert images == [
            Image(
                file_path="/cloud_run.tf",
                name="gcr.io/cloudrun/hello",
                start_line=1,
                end_line=17,
                related_resource_id="/cloud_run.tf:None",
            ),
        ]

    def test_extract_images_from_resources_with_no_image(self):
        # given
        resource = {
            "file_path_": "/cloud_run.tf",
            "__end_line__": 17,
            "__start_line__": 1,
            "template": {
                "spec": {
                    "containers": {
                        "working_dir": "/tmp",
                    }
                }
            },
            "resource_type": "google_cloud_run_service",
        }
        self.graph = set_graph_by_graph_framework(self.graph_framework)
        add_vertices_to_graph_by_graph_framework(self.graph_framework, resource, self.graph)

        # when
        with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': self.graph_framework}):
            gcp_provider = GcpTerraformProvider(graph_connector=self.graph)
            images = gcp_provider.extract_images_from_resources()

        # then
        assert not images


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.NetworkInterfaceEnableIPForwarding import check
from checkov.common.models.enums import CheckResult


class TestNetworkInterfaceEnableIPForwarding(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_network_interface" "example" {
                  name                = "example-nic"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                
                  ip_configuration {
                    name                          = "internal"
                    subnet_id                     = azurerm_subnet.example.id
                    private_ip_address_allocation = "Dynamic"
                  }        
                  enable_ip_forwarding = true
                }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_network_interface']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_network_interface" "example" {
                  name                = "example-nic"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                
                  ip_configuration {
                    name                          = "internal"
                    subnet_id                     = azurerm_subnet.example.id
                    private_ip_address_allocation = "Dynamic"
                  }
                  enable_ip_forwarding = false
                }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_network_interface']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_no_param(self):
        hcl_res = hcl2.loads("""
                resource "azurerm_network_interface" "example" {
                  name                = "example-nic"
                  location            = azurerm_resource_group.example.location
                  resource_group_name = azurerm_resource_group.example.name
                
                  ip_configuration {
                    name                          = "internal"
                    subnet_id                     = azurerm_subnet.example.id
                    private_ip_address_allocation = "Dynamic"
                  }
                }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_network_interface']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import os

from parameterized import parameterized_class

from tests.graph_utils.utils import PARAMETERIZED_GRAPH_FRAMEWORKS
from tests.terraform.graph.checks_infra.test_base import TestBaseSolver

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))


@parameterized_class(PARAMETERIZED_GRAPH_FRAMEWORKS)
class TestNotEndingWithSolver(TestBaseSolver):
    def setUp(self):
        self.checks_dir = TEST_DIRNAME
        super(TestNotEndingWithSolver, self).setUp()

    def test_ami_ending_with(self):
        root_folder = '../../../resources/public_virtual_machines'
        check_id = "AmiEndingWith"
        should_pass = ['aws_instance.with_closed_def_security_groups', 'aws_instance.with_open_security_groups', 'aws_instance.with_subnet_public', 'aws_instance.with_subnet_not_public',]
        should_fail = ['aws_instance.with_open_def_security_groups']
        expected_results = {check_id: {"should_pass": should_pass, "should_fail": should_fail}}

        self.run_test(root_folder=root_folder, expected_results=expected_results, check_id=check_id)



import re
"""
This file provides utility functions for handling OIDC-related operations, particularly for GitHub repositories.

Constants:
    gh_repo_regex (re.Pattern): A regular expression pattern that matches GitHub repository paths.
        # Matches patterns like: "owner/repo", "${var}/repo", "org.name/repo"
        # Allows for variable substitution syntax ${} and organization names with dots

    gh_abusable_claims (list): A list of GitHub OIDC claims that could potentially be abused in security contexts.
"""
gh_repo_regex = re.compile(r"(\$\{)?[a-zA-Z0-9_-]+(\.[a-zA-Z0-9_-]+)*(\})?/[^/]+")
gh_abusable_claims = ["workflow", "environment", "ref", "context", "head_ref", "base_ref"]
gh_sub_condition = re.compile(r"^token\.actions\.githubusercontent\.com(?:/[a-zA-Z0-9_-]+)?:sub$")



import os

from checkov.cloudformation.runner import Runner
from checkov.common.graph.graph_builder import CustomAttributes


def test_connected_node_in_results_is_never_tuple():
    """
    Validates we correctly update the connected_node.
    It cannot be a tuple, as this is unserializable to json.
    """
    file_dir = os.path.dirname(__file__)
    dir_path = os.path.join(file_dir, f"resources", "ALBRedirectHTTPtoHTTPS")
    external_checks_dir = os.path.join(os.path.dirname(os.path.dirname(dir_path)), f"test_checks")

    runner = Runner()
    report = runner.run(dir_path, external_checks_dir=[external_checks_dir])

    all_results = report.failed_checks + report.passed_checks + report.skipped_checks
    for result in all_results:
        if "entity" in result.check_result:
            entity = result.check_result.get("entity", {})
            connected_node = entity.get(CustomAttributes.CONNECTED_NODE)
            if connected_node:
                if isinstance(connected_node, dict):
                    dict_keys = list(connected_node.keys())
                    assert not any([isinstance(key, tuple) for key in dict_keys])



import unittest

import hcl2
from checkov.terraform.checks.resource.github.PrivateRepo import check
from checkov.common.models.enums import CheckResult


class TestPrivateRepo(unittest.TestCase):

    def test_success_private_true(self):
        hcl_res = hcl2.loads("""
        resource "github_repository" "test" {
            description   = "test repo"
            name          = "test"
            private       = true
        }
        """)
        resource_conf = hcl_res['resource'][0]['github_repository']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_visibility_private(self):
        hcl_res = hcl2.loads("""
        resource "github_repository" "test" {
            description   = "test repo"
            name          = "test"
            visibility    = "private"
        }
        """)
        resource_conf = hcl_res['resource'][0]['github_repository']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success_visibility_internal(self):
        hcl_res = hcl2.loads("""
        resource "github_repository" "test" {
            description   = "test repo"
            name          = "test"
            visibility    = "internal"
        }
        """)
        resource_conf = hcl_res['resource'][0]['github_repository']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure_private_false(self):
        hcl_res = hcl2.loads("""
        resource "github_repository" "test" {
            description   = "test repo"
            name          = "test"
            private       = false
        }
        """)
        resource_conf = hcl_res['resource'][0]['github_repository']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_default(self):
        hcl_res = hcl2.loads("""
        resource "github_repository" "test" {
            description   = "test repo"
            name          = "test"
        }
        """)
        resource_conf = hcl_res['resource'][0]['github_repository']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_visibility_public(self):
        hcl_res = hcl2.loads("""
        resource "github_repository" "test" {
            description   = "test repo"
            name          = "test"
            visibility    = "public"
        }
        """)
        resource_conf = hcl_res['resource'][0]['github_repository']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

if __name__ == '__main__':
    unittest.main()



from __future__ import annotations
from typing import Any

from checkov.common.checks.base_check_registry import BaseCheckRegistry

from checkov.common.runners.object_runner import Runner


# Since ObjectRunner is an abstract class, we can't instantiate it.
# This class is used only for testing by filling up the abstract implementations.
class ObjectRunnerImplementedAbstractions(Runner):
    def _parse_file(
            self, f: str
    ) -> tuple[dict[str, Any] | list[dict[str, Any]], list[tuple[int, str]]] | None:
        return None

    def get_start_end_lines(self, end: int, result_config: dict[str, Any], start: int) -> tuple[int, int]:
        return 1, 1

    def import_registry(self) -> BaseCheckRegistry:
        return BaseCheckRegistry("")


def test_get_jobs() -> None:
    string_step_definition = {
        "name": "String step",
        "jobs": {
            "install-dbx": {
                "name": "Install dependencies and project in dev mode",
                "steps": {
                    "run": "pip install dbx\n",
                    "__startline__": 185,
                    "__endline__": 188
                },
                "__startline__": 183,
                "__endline__": 188
            },
            "__startline__": 29,
            "__endline__": 346
        },
        "__startline__": 1,
        "__endline__": 346
    }

    end_line_to_job_name_dict = ObjectRunnerImplementedAbstractions()._get_jobs(string_step_definition)

    assert end_line_to_job_name_dict[188] == "install-dbx"



import unittest

from checkov.common.models.enums import CheckResult
from checkov.terraform.checks.resource.aws.DocDBEncryption import check


class TestDocDBEncryption(unittest.TestCase):

    def test_failure(self):
        resource_conf = {"cluster_identifier": "my-docdb-cluster",
                         "storage_encrypted": False}
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        resource_conf = {"cluster_identifier": "my-docdb-cluster",
                         "storage_encrypted": True}

        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_transfer as transfer

class MyTransferServerStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Transfer Server with EndpointType set to VPC
        transfer.CfnServer(
            self, 'MyTransferServer',
            endpoint_type='VPC',
            # Other properties as needed for your Transfer Server
        )

app = core.App()
MyTransferServerStack(app, "MyTransferServerStack")
app.synth()


class MyTransferServerStack2(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define Transfer Server with EndpointType set to VPC
        transfer.CfnServer(
            self, 'MyTransferServer',
            endpoint_type='VPC_ENDPOINT',
            # Other properties as needed for your Transfer Server
        )

app = core.App()
MyTransferServerStack2(app, "MyTransferServerStack2")
app.synth()



from aws_cdk import core
from aws_cdk import aws_cloudfront as cloudfront

class MyCloudFrontDistributionStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define CloudFront Distribution with logging settings
        distribution = cloudfront.CfnDistribution(
            self, 'MyCloudFrontDistribution',
            distribution_config=cloudfront.CfnDistribution.DistributionConfigProperty(
                logging=cloudfront.CfnDistribution.LoggingProperty(
                    bucket='arn:aws:s3:::my-cloudfront-logs-bucket'  # Replace with your S3 bucket ARN
                ),
                # Other distribution configuration properties
            )
        )

app = core.App()
MyCloudFrontDistributionStack(app, "MyCloudFrontDistributionStack")
app.synth()



import os
import unittest

from checkov.cloudformation.checks.resource.aws.AthenaWorkgroupConfiguration import check
from checkov.cloudformation.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestAthenaWorkgroupConfiguration(unittest.TestCase):

    def test_summary(self):
        runner = Runner()
        current_dir = os.path.dirname(os.path.realpath(__file__))

        # There is some internal conflicts on "Tags" in the Athena WG Docs. And corresponding specs.
        # https://docs.aws.amazon.com/AWSCloudFormation/latest/UserGuide/aws-resource-athena-workgroup.html
        # This may mean that the "Tags" in this test data needs to correspondingly change when CF / CF Lint are updated in the future.
        # CF Lint Issue: https://github.com/aws-cloudformation/cfn-python-lint/issues/1577
        test_files_dir = current_dir + "/example_AthenaWorkgroupConfiguration"
        report = runner.run(root_folder=test_files_dir,runner_filter=RunnerFilter(checks=[check.id]))
        summary = report.get_summary()

        self.assertEqual(summary['passed'], 1)
        self.assertEqual(summary['failed'], 2)
        self.assertEqual(summary['skipped'], 0)
        self.assertEqual(summary['parsing_errors'], 0)


if __name__ == '__main__':
    unittest.main()



import unittest

from checkov.terraform.checks.resource.aws.PasswordPolicyExpiration import check
from checkov.common.models.enums import CheckResult


class TestPasswordPolicyExpiration(unittest.TestCase):

    def test_success(self):
        resource_conf = {
            "minimum_password_length": [15],
            "require_lowercase_characters": [True],
            "require_numbers": [True],
            "require_uppercase_characters": [True],
            "require_symbols": [True],
            "allow_users_to_change_password": [True],
            "max_password_age": [90]

        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure(self):
        resource_conf = {
            "minimum_password_length": [15],
            "require_lowercase_characters": [True],
            "require_numbers": [True],
            "require_uppercase_characters": [True],
            "require_symbols": [True],
            "allow_users_to_change_password": [True],
            "max_password_age": [91]

        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure_on_missing_property(self):
        resource_conf = {
            "minimum_password_length": [15],
            "require_lowercase_characters": [True],
            "require_numbers": [True],
            "require_uppercase_characters": [True],
            "require_symbols": [True],
            "allow_users_to_change_password": [True],
        }
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.FunctionAppDisallowCORS import check
from checkov.common.models.enums import CheckResult


class TestFunctionAppDisallowCORS(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_function_app" "example" {
              name                       = "test-azure-functions"
              location                   = azurerm_resource_group.example.location
              resource_group_name        = azurerm_resource_group.example.name
              app_service_plan_id        = azurerm_app_service_plan.example.id
              storage_account_name       = azurerm_storage_account.example.name
              storage_account_access_key = azurerm_storage_account.example.primary_access_key
              site_config {
                cors {
                    allowed_origins = ["*"]
                }
              }
            }
                """)
        resource_conf = hcl_res['resource'][0]['azurerm_function_app']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success1(self):
        hcl_res = hcl2.loads("""
              resource "azurerm_function_app" "example" {
                name                       = "test-azure-functions"
                location                   = azurerm_resource_group.example.location
                resource_group_name        = azurerm_resource_group.example.name
                app_service_plan_id        = azurerm_app_service_plan.example.id
                storage_account_name       = azurerm_storage_account.example.name
                storage_account_access_key = azurerm_storage_account.example.primary_access_key
                site_config {
                  cors {
                      allowed_origins = ["192.0.0.1"]
                  }
                }
              }
                  """)
        resource_conf = hcl_res['resource'][0]['azurerm_function_app']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_success2(self):
        hcl_res = hcl2.loads("""
              resource "azurerm_function_app" "example" {
                name                       = "test-azure-functions"
                location                   = azurerm_resource_group.example.location
                resource_group_name        = azurerm_resource_group.example.name
                app_service_plan_id        = azurerm_app_service_plan.example.id
                storage_account_name       = azurerm_storage_account.example.name
                storage_account_access_key = azurerm_storage_account.example.primary_access_key
              }
                  """)
        resource_conf = hcl_res['resource'][0]['azurerm_function_app']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import os
import unittest

from checkov.openapi.checks.resource.generic.ClearTextAPIKey import check
from checkov.openapi.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestClearTextAPIKey(unittest.TestCase):
    def test_summary(self):
        # given
        current_dir = os.path.dirname(os.path.realpath(__file__))
        test_files_dir = current_dir + "/example_ClearTextAPIKey"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "/pass.yaml",
            "/pass.json",
            "/pass2.yaml",
            "/pass2.json",
            "/pass3.yaml",
            "/pass3.json",
            "/pass4.yaml",
            "/pass4.json",
            "/pass5.yaml",
            "/pass5.json",
            "/pass6.yaml",
            "/pass6.json",
        }
        failing_resources = {
            "/fail.yaml",
            "/fail.json",
            "/fail2.yaml",
            "/fail2.json",
            "/fail3.yaml",
            "/fail3.json",
            "/fail4.yaml",
            "/fail4.json",
            "/fail5.yaml",
            "/fail5.json",
        }

        passed_check_resources = {c.file_path for c in report.passed_checks}
        failed_check_resources = {c.file_path for c in report.failed_checks}

        self.assertEqual(summary["passed"], len(passing_resources))
        self.assertEqual(summary["failed"], len(failing_resources))
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == "__main__":
    unittest.main()


from aws_cdk import core
from aws_cdk import aws_docdb as docdb

class MyDocDBClusterStack(core.Stack):

    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Create an Amazon DocumentDB cluster with storage encryption disabled
        docdb_cluster = docdb.CfnDBCluster(
            self,
            "MyDocDBCluster",
            db_cluster_identifier="my-docdb-cluster",
            master_username="admin",
            master_user_password="mypassword", # checkov:skip=CKV_SECRET_6 test secret
            availability_zones=["us-east-1a", "us-east-1b"],  # Specify the availability zones
            port=27017,  # Specify the port as needed
        )



from pathlib import Path

from checkov.bicep.runner import Runner
from checkov.arm.checks.resource.MonitorLogProfileCategories import check
from checkov.runner_filter import RunnerFilter


def test_examples():
    # given
    test_files_dir = Path(__file__).parent / "example_MonitorLogProfileCategories"

    # when
    report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

    # then
    summary = report.get_summary()

    passing_resources = {
        "Microsoft.Insights/logprofiles.pass",
    }

    failing_resources = {
        "Microsoft.Insights/logprofiles.fail",
    }

    passed_check_resources = {c.resource for c in report.passed_checks}
    failed_check_resources = {c.resource for c in report.failed_checks}

    assert summary["passed"] == 1
    assert summary["failed"] == 1
    assert summary["skipped"] == 0
    assert summary["parsing_errors"] == 0

    assert passed_check_resources == passing_resources
    assert failed_check_resources == failing_resources



from typing import Any


class Image:
    attrs: dict[str, Any]
    @property
    def id(self) -> str: ...  # is actually defined in its parent class 'Model'
    @property
    def short_id(self) -> str: ...


class ImageCollection:
    def get(self, name: str) -> Image: ...
    def pull(self, repository: str, tag: str | None= ..., all_tags: bool = ..., **kwargs: Any) -> Image: ...



from __future__ import annotations

import os
from pathlib import Path
from unittest import mock
from urllib.parse import quote_plus

import responses
from aioresponses import aioresponses
from yarl import URL

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.code_categories import CodeCategoryType
from checkov.common.bridgecrew.severities import Severities, BcSeverities
from checkov.common.models.enums import CheckResult
from checkov.dockerfile.runner import Runner as Dockerfile_Runner
from checkov.kubernetes.runner import Runner as Kubernetes_Runner
from checkov.runner_filter import RunnerFilter
from checkov.sca_image.runner import Runner
from .conftest import DOCKERFILE_EXAMPLES_DIR, KUBERNETES_EXAMPLES_DIR
from .mocks import mock_scan_empty, mock_scan_image


def test_image_referencer_trigger_image_flow_calls(mock_bc_integration, image_name, cached_scan_result):
    # given
    image_id_encoded = quote_plus(f"image:{image_name}")

    response_json = {
        "violations": [
            {
                "name": "readline",
                "version": "8.1.2-r0",
                "license": "Apache-2.0",
                "policy": "BC_LIC_1",
                "status": "OPEN"
            },
            {
                "name": "libnsl",
                "version": "2.0.0-r0",
                "license": "Apache-2.0",
                "policy": "BC_LIC_1",
                "status": "COMPLIANT"
            },
        ]
    }
    with aioresponses() as mock_response:
        mock_response.post(
            url=mock_bc_integration.api_url + "/api/v1/vulnerabilities/packages/get-licenses-violations",
            payload=response_json,
            status=200
        )
        mock_response.get(
            url=URL(mock_bc_integration.api_url + f"/api/v1/vulnerabilities/scan-results/{image_id_encoded}",
                    encoded=True),
            payload=cached_scan_result,
            status=200,
        )

        # when
        reports = Dockerfile_Runner().run(root_folder=str(DOCKERFILE_EXAMPLES_DIR),
                                          runner_filter=RunnerFilter(run_image_referencer=True))

        sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    # then
    assert len(sca_image_report.failed_checks) == 4
    assert len(sca_image_report.passed_checks) == 1


def test_runner_honors_enforcement_rules(mock_bc_integration, image_name2, cached_scan_result):
    # given
    image_id_encoded = quote_plus(f"image:{image_name2}")

    response_json = {
        "violations": [
            {
                "name": "readline",
                "version": "8.1.2-r0",
                "license": "Apache-2.0",
                "policy": "BC_LIC_1",
                "status": "OPEN"
            },
            {
                "name": "libnsl",
                "version": "2.0.0-r0",
                "license": "Apache-2.0",
                "policy": "BC_LIC_1",
                "status": "COMPLIANT"
            },
        ]
    }

    with aioresponses() as mock_response:
        mock_response.post(
            url=mock_bc_integration.api_url + "/api/v1/vulnerabilities/packages/get-licenses-violations",
            payload=response_json,
            status=200
        )
        mock_response.get(
            url=URL(mock_bc_integration.api_url + f"/api/v1/vulnerabilities/scan-results/{image_id_encoded}", encoded=True),
            payload=cached_scan_result,
            status=200,
        )

        # when
        runner_filter = RunnerFilter(use_enforcement_rules=True, run_image_referencer=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        runner_filter.enforcement_rule_configs = {
            CheckType.KUBERNETES: Severities[BcSeverities.OFF],
            CheckType.SCA_IMAGE: {
                CodeCategoryType.LICENSES: Severities[BcSeverities.OFF],
                CodeCategoryType.VULNERABILITIES: Severities[BcSeverities.OFF]
            }
        }

        reports = Kubernetes_Runner().run(root_folder=str(KUBERNETES_EXAMPLES_DIR), runner_filter=runner_filter)
        sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

        summary = sca_image_report.get_summary()
    # then
    assert summary["passed"] == 0
    assert summary["failed"] == 0
    assert summary["skipped"] == 5
    assert summary["parsing_errors"] == 0


def test_run(sca_image_report):
    # given
    report = sca_image_report

    # then
    rootless_path = "path/to/Dockerfile"
    assert report.check_type == "sca_image"
    assert report.resources == {f'{rootless_path} (sha256:123456).pcre2', f'{rootless_path} (sha256:123456).perl'}

    assert len(report.passed_checks) == 1
    assert len(report.failed_checks) == 3
    assert len(report.skipped_checks) == 1
    assert len(report.parsing_errors) == 0

    cve_record = next((c for c in report.failed_checks if
                       c.resource == f"{rootless_path} (sha256:123456).pcre2" and c.check_name == "SCA package scan"),
                      None)
    assert cve_record is not None
    assert cve_record.bc_check_id == "BC_CVE_2022_1587"
    assert cve_record.check_id == "CKV_CVE_2022_1587"
    assert cve_record.check_class == "checkov.common.bridgecrew.vulnerability_scanning.image_scanner.ImageScanner"  # not the real one
    assert cve_record.check_name == "SCA package scan"
    assert cve_record.check_result == {"result": CheckResult.FAILED}
    assert cve_record.code_block == [(0, "pcre2: 10.39-3build1")]
    assert cve_record.description == (
        "An out-of-bounds read vulnerability was discovered in the PCRE2 library in the get_recurse_data_length() function of the pcre2_jit_compile.c file. "
        "This issue affects recursions in JIT-compiled regular expressions caused by duplicate data transfers."
    )
    assert cve_record.file_abs_path == f"/{rootless_path}"
    assert cve_record.file_line_range == [0, 0]
    assert cve_record.file_path == f"/{rootless_path} (sha256:123456)"
    assert cve_record.repo_file_path == f"/{rootless_path}"
    assert cve_record.resource == f"{rootless_path} (sha256:123456).pcre2"
    assert cve_record.severity == Severities[BcSeverities.LOW]
    assert cve_record.short_description == "CVE-2022-1587 - pcre2: 10.39-3build1"
    assert cve_record.vulnerability_details["lowest_fixed_version"] == "N/A"
    assert cve_record.vulnerability_details["fixed_versions"] == []

    assert {"licenses", "package_type"} <= cve_record.vulnerability_details.keys()
    assert cve_record.vulnerability_details["licenses"] == "Apache-2.0"
    assert cve_record.vulnerability_details["package_type"] == "os"

    skipped_record = report.skipped_checks[0]
    assert skipped_record.check_result == {"result": CheckResult.SKIPPED,
                                           'suppress_comment': 'CVE-2022-1586 is skipped'}
    assert skipped_record.short_description == "CVE-2022-1586 - pcre2: 10.39-3build1"

    # making sure extra-resources (a scanned packages without cves) also have licenses - this data will be printed
    # as part of the BON report.
    extra_resource = next((c for c in report.extra_resources if c.resource == f"{rootless_path} (sha256:123456).bzip2"),
                          None)
    assert extra_resource is not None
    assert "licenses" in extra_resource.vulnerability_details
    assert extra_resource.vulnerability_details["licenses"] == "Unknown"

    license_resource = next((c for c in report.failed_checks if c.check_name == "SCA license" if
                             c.resource == f"{rootless_path} (sha256:123456).perl"), None)
    assert license_resource is not None
    print(license_resource.resource)
    assert license_resource.check_id == "BC_LIC_1"
    assert license_resource.bc_check_id == "BC_LIC_1"
    assert license_resource.check_result == {"result": CheckResult.FAILED}

    assert {"package_name", "package_name", "license", "status",
            "policy", "package_type"} <= license_resource.vulnerability_details.keys()
    assert license_resource.vulnerability_details["package_name"] == "perl"
    assert license_resource.vulnerability_details["package_version"] == "5.34.0-3ubuntu1"
    assert license_resource.vulnerability_details["license"] == "Apache-2.0-Fake"
    assert license_resource.vulnerability_details["status"] == "FAILED"
    assert license_resource.vulnerability_details["policy"] == "BC_LIC_1"
    assert license_resource.vulnerability_details["package_type"] == "os"


def test_run_license_policy(mock_bc_integration, image_name, cached_scan_result):
    # given
    image_id_encoded = quote_plus(f"image:{image_name}")

    response_json = {
        "violations": [
            {
                "name": "readline",
                "version": "8.1.2-r0",
                "license": "Apache-2.0",
                "policy": "BC_LIC_1",
                "status": "OPEN"
            },
            {
                "name": "libnsl",
                "version": "2.0.0-r0",
                "license": "Apache-2.0",
                "policy": "BC_LIC_1",
                "status": "COMPLIANT"
            },
        ]
    }
    with aioresponses() as mock_response:
        mock_response.post(
            url=mock_bc_integration.api_url + "/api/v1/vulnerabilities/packages/get-licenses-violations",
            payload=response_json,
            status=200
        )
        mock_response.get(
            url=URL(mock_bc_integration.api_url + f"/api/v1/vulnerabilities/scan-results/{image_id_encoded}", encoded=True),
            payload=cached_scan_result,
            status=200,
        )

        # when
        runner_filter = RunnerFilter(checks=['BC_LIC_1'], run_image_referencer=True)
        reports = Dockerfile_Runner().run(root_folder=str(DOCKERFILE_EXAMPLES_DIR), runner_filter=runner_filter)
        sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)
    # then
    assert not [c for c in sca_image_report.passed_checks + sca_image_report.failed_checks
                if c.check_id.startswith('CKV_CVE')]


@mock.patch('checkov.sca_image.runner.Runner.scan', mock_scan_empty)
@responses.activate
def test_run_with_empty_scan_result(mock_bc_integration):
    # given
    response_json = {
        "violations": [
            {
                "name": "pcre2",
                "version": "10.39-3build1",
                "license": "Apache-2.0",
                "policy": "BC_LIC_1",
                "status": "COMPLIANT"
            },
            {
                "name": "perl",
                "version": "5.34.0-3ubuntu1",
                "license": "Apache-2.0-Fake",
                "policy": "BC_LIC_1",
                "status": "OPEN"
            },
        ]
    }
    responses.add(
        method=responses.POST,
        url=mock_bc_integration.api_url + "/api/v1/vulnerabilities/packages/get-licenses-violations",
        json=response_json,
        status=200
    )

    runner = Runner()
    runner_filter = RunnerFilter(skip_checks=["CKV_CVE_2022_1586"])
    # when
    dockerfile_path = "/Users/ipeleg/Work/checkov/tests/sca_image/examples/dockerfile/Dockerfile"
    image_id = "sha256:123456"
    report = runner.run(root_folder=DOCKERFILE_EXAMPLES_DIR, runner_filter=runner_filter,
                        dockerfile_path=dockerfile_path, image_id=image_id)

    # then
    assert report.check_type == "sca_image"
    assert report.resources == set()

    assert len(report.passed_checks) == 0
    assert len(report.failed_checks) == 0
    assert len(report.skipped_checks) == 0
    assert len(report.parsing_errors) == 0


@mock.patch.dict(os.environ, {"CKV_IGNORE_HIDDEN_DIRECTORIES": "false"})
@mock.patch('checkov.sca_image.runner.Runner.get_image_cached_results', mock_scan_image)
def test_run_with_image_cached_reports_env(mock_bc_integration, image_name, cached_scan_result2):
    image_id_encoded = quote_plus(f"image:{image_name}")

    with aioresponses() as mock_response:
        mock_response.get(
            url=URL(mock_bc_integration.api_url + f"/api/v1/vulnerabilities/scan-results/{image_id_encoded}", encoded=True),
            payload=cached_scan_result2,
            status=200,
        )

        runner_filter = RunnerFilter(run_image_referencer=True)
        reports = Dockerfile_Runner().run(root_folder=str(DOCKERFILE_EXAMPLES_DIR), runner_filter=runner_filter)
        sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 1
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0
    assert len(sca_image_report.image_cached_results) == 1


@mock.patch.dict(os.environ, {"CHECKOV_CREATE_SCA_IMAGE_REPORTS_FOR_IR": "False"})
@mock.patch.dict(os.environ, {"CKV_IGNORE_HIDDEN_DIRECTORIES": "false"})
@mock.patch('checkov.sca_image.runner.Runner.get_image_cached_results', mock_scan_image)
def test_run_with_image_cached_reports_and_without_sca_reports_env(mock_bc_integration, image_name,
                                                                   cached_scan_result2):
    image_id_encoded = quote_plus(f"image:{image_name}")
    with aioresponses() as mock_response:
        mock_response.get(
            url=URL(mock_bc_integration.api_url + f"/api/v1/vulnerabilities/scan-results/{image_id_encoded}", encoded=True),
            payload=cached_scan_result2,
            status=200,
        )

        runner_filter = RunnerFilter(run_image_referencer=True)
        reports = Dockerfile_Runner().run(root_folder=str(DOCKERFILE_EXAMPLES_DIR), runner_filter=runner_filter)
        sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 1
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0
    assert len(sca_image_report.image_cached_results) == 1


@responses.activate
def test_run_with_error_from_scan_results(mock_bc_integration, image_name2, cached_scan_result3):
    image_id_encoded = quote_plus(f"image:{image_name2}")

    responses.add(
        method=responses.GET,
        url=mock_bc_integration.api_url + f"/api/v1/vulnerabilities/scan-results/{image_id_encoded}",
        json=cached_scan_result3,
        status=500,
    )

    runner = Runner()
    runner_filter = RunnerFilter(skip_checks=["CKV_CVE_2022_1586"])
    # when
    image_id = "sha256:123456"
    report = runner.run(root_folder=DOCKERFILE_EXAMPLES_DIR, runner_filter=runner_filter, image_id=image_id,
                        files=[".github/workflows/vulnerable_container.yaml"])

    assert len(report.passed_checks) == 0
    assert len(report.failed_checks) == 0
    assert len(report.skipped_checks) == 0
    assert len(report.parsing_errors) == 0
    assert len(report.image_cached_results) == 0



import pprint
from typing import List, Tuple
from unittest import TestCase

from checkov.common.graph.graph_builder.graph_components.attribute_names import CustomAttributes
from checkov.terraform.graph_builder.graph_components.block_types import BlockType
from checkov.terraform.graph_builder.utils import get_referenced_vertices_in_value, \
    replace_map_attribute_access_with_dot, generate_possible_strings_from_wildcards, \
    attribute_has_nested_attributes
from checkov.terraform.graph_builder.variable_rendering.vertex_reference import TerraformVertexReference


class TestUtils(TestCase):
    def test_find_non_literal_values(self):
        aliases = {'aws': {CustomAttributes.BLOCK_TYPE: BlockType.PROVIDER}}
        str_value = 'aws.east1'
        expected = [TerraformVertexReference(BlockType.PROVIDER, ['aws', 'east1'], 'aws.east1')]
        self.assertEqual(expected, get_referenced_vertices_in_value(str_value, aliases, []))

        str_values = [
            'var.x',
            'format("-%s", var.x)',
            '../child',
            'aws_instance.example.id',
            'bc_c_${var.customer_name}',
            'aws iam delete-role --role-name ${local.role_name} --profile ${var.profile} --region ${var.region}',
            'length(aws_vpc.main) > 0 ? aws_vpc.main[0].cidr_block : ${var.x}',
        ]
        expected = [
            [TerraformVertexReference(BlockType.VARIABLE, ['x'], 'var.x')],
            [TerraformVertexReference(BlockType.VARIABLE, ['x'], 'var.x')],
            [],
            [TerraformVertexReference(BlockType.RESOURCE, ['aws_instance.example', 'id'], 'aws_instance.example.id')],
            [TerraformVertexReference(BlockType.VARIABLE, ['customer_name'], 'var.customer_name')],
            [TerraformVertexReference(BlockType.LOCALS, ['role_name'], 'local.role_name'), TerraformVertexReference(BlockType.VARIABLE, ['profile'], 'var.profile'), TerraformVertexReference(BlockType.VARIABLE, ['region'], 'var.region')],
            [TerraformVertexReference(BlockType.RESOURCE, ['aws_vpc.main'], 'aws_vpc.main'), TerraformVertexReference(BlockType.RESOURCE, ['aws_vpc.main', 'cidr_block'], 'aws_vpc.main.cidr_block'), TerraformVertexReference(BlockType.VARIABLE, ['x'], 'var.x')],
        ]

        for i in range(0, len(str_values)):
            self.assertEqual(expected[i], get_referenced_vertices_in_value(str_values[i], aliases, ['aws_vpc', 'aws_instance']))

    def test_replace_map_attribute_access_with_dot(self):
        str_value = 'data.aws_availability_zones["available"].names[1]'
        replace_map_attribute_access_with_dot(str_value)
        self.assertEqual('data.aws_availability_zones.available.names[1]', replace_map_attribute_access_with_dot(str_value))

        str_value = 'data.aws_availability_zones[0].names[1]'
        replace_map_attribute_access_with_dot(str_value)
        self.assertEqual('data.aws_availability_zones[0].names[1]', replace_map_attribute_access_with_dot(str_value))


    def test_generate_possible_strings_from_wildcards(self):
        origin_string = "a.*.b.*.c.*"
        expected_results = [
            "a.0.b.0.c.0",
            "a.0.b.1.c.0",
            "a.1.b.0.c.0",
            "a.0.b.0.c.1",
            "a.1.b.1.c.0",
            "a.1.b.0.c.1",
            "a.0.b.1.c.1",
            "a.1.b.1.c.1",
            "a.b.c",
        ]
        expected_results.sort()
        results = generate_possible_strings_from_wildcards(origin_string=origin_string, max_entries=2)
        results.sort()
        self.assertEqual(expected_results, results)

    def test_find_var_blocks(self):
        cases: List[Tuple[str, List[TerraformVertexReference]]] = [
            (
                "${local.one}",
                [
                    TerraformVertexReference(BlockType.LOCALS, sub_parts=["one"], origin_value="local.one")
                ]
            ),
            (
                "${local.NAME[foo]}-${local.TAIL}${var.gratuitous_var_default}",
                [
                    TerraformVertexReference(BlockType.LOCALS, sub_parts=["NAME"], origin_value="local.NAME"),
                    TerraformVertexReference(BlockType.LOCALS, sub_parts=["TAIL"], origin_value="local.TAIL"),
                    TerraformVertexReference(BlockType.VARIABLE, sub_parts=["gratuitous_var_default"], origin_value="var.gratuitous_var_default"),
                ]
            ),
            # Ordered returning of sub-vars and then outer var.
            (
                "${merge(local.common_tags,local.common_data_tags,{'Name': 'my-thing-${var.ENVIRONMENT}-${var.REGION}'})}",
                [
                    TerraformVertexReference(BlockType.LOCALS, sub_parts=["common_tags"], origin_value="local.common_tags"),
                    TerraformVertexReference(BlockType.LOCALS, sub_parts=["common_data_tags"], origin_value="local.common_data_tags"),
                    TerraformVertexReference(BlockType.VARIABLE, sub_parts=["ENVIRONMENT"],
                                    origin_value="var.ENVIRONMENT"),
                    TerraformVertexReference(BlockType.VARIABLE, sub_parts=["REGION"],
                                    origin_value="var.REGION"),
                ],
            ),
            (
                "${merge(${local.common_tags},${local.common_data_tags},{'Name': 'my-thing-${var.ENVIRONMENT}-${var.REGION}'})}",
                [
                    TerraformVertexReference(BlockType.LOCALS, sub_parts=["common_tags"], origin_value="local.common_tags"),
                    TerraformVertexReference(BlockType.LOCALS, sub_parts=["common_data_tags"],
                                    origin_value="local.common_data_tags"),
                    TerraformVertexReference(BlockType.VARIABLE, sub_parts=["ENVIRONMENT"],
                                    origin_value="var.ENVIRONMENT"),
                    TerraformVertexReference(BlockType.VARIABLE, sub_parts=["REGION"],
                                    origin_value="var.REGION"),
                ],
            ),
            (
                '${merge(var.tags, map("Name", "${var.name}", "data_classification", "none"))}',
                [
                    TerraformVertexReference(BlockType.VARIABLE, sub_parts=["tags"],
                                    origin_value="var.tags"),
                    TerraformVertexReference(BlockType.VARIABLE, sub_parts=["name"],
                                    origin_value="var.name"),
                ]
            ),
            (
                '${var.metadata_http_tokens_required ? "required" : "optional"}',
                [
                    TerraformVertexReference(BlockType.VARIABLE, sub_parts=["metadata_http_tokens_required"],
                                    origin_value="var.metadata_http_tokens_required"),
                ]
            ),
            (
                '${local.NAME[${module.bucket.bucket_name}]}-${local.TAIL}${var.gratuitous_var_default}',
                [
                    TerraformVertexReference(BlockType.LOCALS, sub_parts=["NAME"],
                                    origin_value="local.NAME"),
                    TerraformVertexReference(BlockType.MODULE, sub_parts=["bucket", "bucket_name"],
                                    origin_value="module.bucket.bucket_name"),
                    TerraformVertexReference(BlockType.LOCALS, sub_parts=["TAIL"],
                                    origin_value="local.TAIL"),
                    TerraformVertexReference(BlockType.VARIABLE, sub_parts=["gratuitous_var_default"],
                                    origin_value="var.gratuitous_var_default"),
                ]
            ),
        ]
        for case in cases:
            actual = get_referenced_vertices_in_value(value=case[0], aliases={}, resources_types=[])
            assert actual == case[1], \
                f"Case \"{case[0]}\" failed ❌:\n" \
                f"  Expected: \n{pprint.pformat([str(c) for c in case[1]], indent=2)}\n\n" \
                f"  Actual: \n{pprint.pformat([str(c) for c in actual], indent=2)}"
            print(f"Case \"{case[0]}: ✅")

    def test__attribute_has_nested_attributes_dictionary(self):
        attributes = {'name': ['${var.lb_name}'], 'internal': [True], 'security_groups': ['${var.lb_security_group_ids}'], 'subnets': ['${var.subnet_id}'], 'enable_deletion_protection': [True], 'tags': {'Terraform': True, 'Environment': 'sophi-staging'}, 'resource_type': ['aws_alb'], 'tags.Terraform': True, 'tags.Environment': 'sophi-staging'}
        self.assertTrue(attribute_has_nested_attributes(attribute_key='tags', attributes=attributes))
        self.assertFalse(attribute_has_nested_attributes(attribute_key='name', attributes=attributes))
        self.assertFalse(attribute_has_nested_attributes(attribute_key='tags.Environment', attributes=attributes))

    def test__attribute_has_nested_attributes_list(self):
        attributes = {'most_recent': [True], 'filter': [{'name': 'name', 'values': ['amzn-ami-hvm-*-x86_64-gp2']}, {'name': 'owner-alias', 'values': ['amazon']}], 'filter.0': {'name': 'name', 'values': ['amzn-ami-hvm-*-x86_64-gp2']}, 'filter.0.name': 'name', 'filter.0.values': ['amzn-ami-hvm-*-x86_64-gp2'], 'filter.0.values.0': 'amzn-ami-hvm-*-x86_64-gp2', 'filter.1': {'name': 'owner-alias', 'values': ['amazon']}, 'filter.1.name': 'owner-alias', 'filter.1.values': ['amazon'], 'filter.1.values.0': 'amazon'}
        self.assertTrue(attribute_has_nested_attributes(attribute_key='filter', attributes=attributes))
        self.assertTrue(attribute_has_nested_attributes(attribute_key='filter.1.values', attributes=attributes))
        self.assertFalse(attribute_has_nested_attributes(attribute_key='filter.1.values.0', attributes=attributes))



import pytest

from checkov.common.graph.graph_builder.graph_components.blocks import Block


@pytest.mark.parametrize("input_key, expected_key", (
        ("a.b", "$.a.b"),
        ("a.0", "$.a[0]"),
        ("a.0.b.1.2.c", "$.a[0].b[1][2].c"),
        ("a.0./mock-part-of-key.d.e.1", "$.a[0].\"/mock-part-of-key\".d.e[1]"),
        ("a.0.Fn::Region.d.e.1", "$.a[0].\"Fn::Region\".d.e[1]")
))
def test__get_jsonpath_key(input_key: str, expected_key: str) -> None:
    result = Block._get_jsonpath_key(input_key)
    assert result == expected_key



import json
import os
from pathlib import Path

from cyclonedx.model.component import Component, ComponentType
from cyclonedx.model.vulnerability import VulnerabilitySeverity
from packageurl import PackageURL

from checkov.common.output.extra_resource import ExtraResource
from checkov.common.output.report import Report, CheckType
from pytest_mock import MockerFixture

from checkov.common.output.common import ImageDetails
from checkov.common.output.cyclonedx import CycloneDX
from checkov.common.sca.commons import get_package_lines
from checkov.common.sca.output import create_report_cve_record
from checkov.common.output.record import Record
from checkov.terraform.runner import Runner


def test_valid_cyclonedx_bom():
    # given
    test_file = Path(__file__).parent / "fixtures/main.tf"
    repo_id = "acme/example"
    report = Runner().run(root_folder="", files=[str(test_file)])

    # when
    cyclonedx = CycloneDX(repo_id=repo_id, reports=[report])
    output = cyclonedx.get_xml_output()

    # then
    assert len(cyclonedx.bom.components) == 1

    component = next(iter(cyclonedx.bom.components))

    assert component.name == 'aws_s3_bucket.destination'
    assert component.purl.name == 'aws_s3_bucket.destination'
    assert component.purl.namespace == 'acme/example/main.tf'
    assert component.purl.type == 'terraform'
    assert component.purl.version.startswith('sha1:')
    assert component.type == ComponentType.APPLICATION

    assert len(cyclonedx.bom.vulnerabilities) == 6
    # doesn't matter which vulnerability, they are all unknown for runs without platform connection
    assert next(iter(next(iter(cyclonedx.bom.vulnerabilities)).ratings)).severity == VulnerabilitySeverity.UNKNOWN

    assert "http://cyclonedx.org/schema/bom/1.4" in output

def test_valid_cyclonedx_image_bom():
    # given
    repo_id = 'acme/repo'
    rootless_file_path = "Dockerfile (sha256:123456)"
    file_abs_path = "/path/to/Dockerfile (sha256:123456)"
    check_class = "checkov.common.bridgecrew.vulnerability_scanning.sca_scanner.imageScanner"
    image_details: ImageDetails = ImageDetails(
        distro='Debian GNU/Linux 11 (bullseye)',
        distro_release='bullseye',
        package_types={'curl@7.74.0-1.3+deb11u1': 'os'},
        image_id='ubuntu:latest'
    )
    vulnerability = {
        'id': 'CVE-2022-32207',
        'status': 'fixed in 7.74.0-1.3+deb11u2',
        'cvss': 9.8,
        'vector': 'CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H',
        'description': 'When curl < 7.84.0 saves cookies, alt-svc and hsts data to local files, it makes the operation atomic by finalizing the operation with a rename from a temporary name to the final target file name.In that rename operation, it might accidentally *widen* the permissions for the target file, leaving the updated file accessible to more users than intended.',
        'severity': 'critical',
        'packageName': 'curl',
        'packageVersion': '7.74.0-1.3+deb11u1',
        'link': 'https://security-tracker.debian.org/tracker/CVE-2022-32207',
        'riskFactors': [
            'Attack complexity: low',
            'Attack vector: network',
            'Critical severity',
            'Has fix',
            'Recent vulnerability'
        ],
        'impactedVersions': ['<7.74.0-1.3+deb11u2'],
        'publishedDate': '2022-07-07T16:15:00+03:00',
        'discoveredDate': '2022-08-02T14:05:40+03:00',
        'fixDate': '2022-07-07T16:15:00+03:00'
    }

    record: Record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability,
        licenses="BSD-3-Clause",
        sca_details=image_details,
        package={'package_registry': "https://registry.npmjs.org/", 'is_private_registry': False},
    )
    report = Report(check_type='sca_image')
    report.add_record(record)

    # when
    cyclonedx = CycloneDX(repo_id=repo_id, reports=[report])
    output = cyclonedx.get_xml_output()

    # then
    assert len(cyclonedx.bom.components) == 2
    package_purl = PackageURL(
        name='curl',
        namespace='acme/repo/Dockerfile/debian',
        type='deb',
        version='7.74.0-1.3+deb11u1',
        qualifiers={'distro': 'bullseye'}
    )

    package_component = cyclonedx.bom.get_component_by_purl(purl=package_purl)

    assert package_component is not None
    assert package_component.name == "curl"
    assert package_component.type == ComponentType.LIBRARY
    assert package_component.version == "7.74.0-1.3+deb11u1"
    assert len(package_component.licenses) == 1
    assert next(iter(package_component.licenses)).name == "BSD-3-Clause"

    assert len(cyclonedx.bom.vulnerabilities) == 1
    assert next(iter(next(iter(cyclonedx.bom.vulnerabilities)).ratings)).severity == VulnerabilitySeverity.CRITICAL

    image_purl = PackageURL(
        name='Dockerfile',
        namespace='acme/repo',
        type='oci',
        version='ubuntu:latest',
    )
    image_component = Component(
        name='acme/repo//ubuntu:latest',
        purl=image_purl,
        group=None,
        type=ComponentType.CONTAINER,
        version=''
    )
    assert cyclonedx.bom.has_component(image_component)

    assert "http://cyclonedx.org/schema/bom/1.4" in output


def test_sca_packages_cyclonedx_bom():
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package_2.scanner.Scanner"
    vulnerability_details = {
        "id": "CVE-2019-19844",
        "status": "fixed in 3.0.1, 2.2.9, 1.11.27",
        "cvss": 9.8,
        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
        "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
        "severity": "moderate",
        "packageName": "django",
        "packageVersion": "1.2",
        "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
        "riskFactors": ["Attack complexity: low", "Attack vector: network", "Critical severity", "Has fix"],
        "impactedVersions": ["<1.11.27"],
        "publishedDate": "2019-12-18T20:15:00+01:00",
        "discoveredDate": "2019-12-18T19:15:00Z",
        "fixDate": "2019-12-18T20:15:00+01:00",
    }
    package = {'package_registry': "https://registry.npmjs.org/",
               'is_private_registry': False,
               "linesNumbers": [2, 6]}

    # when
    record = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        licenses='OSI_BDS',
        package=package,
        file_line_range=get_package_lines(package)
    )

    report = Report(CheckType.SCA_PACKAGE)
    report.add_resource(record.resource)
    report.add_record(record)

    report.extra_resources.add(
        ExtraResource(
            file_abs_path=file_abs_path,
            file_path=f"/{rootless_file_path}",
            resource=f'{rootless_file_path}.testpkg',
            vulnerability_details={
                "package_name": "testpkg",
                "package_version": "1.1.1",
                "licenses": "MIT",
            }
        )
    )

    cyclonedx = CycloneDX([report], "repoid/test")
    output = cyclonedx.get_xml_output()

    # then
    assert record.file_line_range == [2, 6]
    assert output


def test_duplicate_sca_packages_cyclonedx_bom():
    # given
    rootless_file_path = "requirements.txt"
    file_abs_path = "/path/to/requirements.txt"
    check_class = "checkov.sca_package_2.scanner.Scanner"
    vulnerability_details = {
        "id": "CVE-2019-19844",
        "status": "fixed in 3.0.1, 2.2.9, 1.11.27",
        "cvss": 9.8,
        "vector": "CVSS:3.1/AV:N/AC:L/PR:N/UI:N/S:U/C:H/I:H/A:H",
        "description": "Django before 1.11.27, 2.x before 2.2.9, and 3.x before 3.0.1 allows account takeover. ...",
        "severity": "moderate",
        "packageName": "django",
        "packageVersion": "1.2",
        "link": "https://nvd.nist.gov/vuln/detail/CVE-2019-19844",
        "riskFactors": ["Attack complexity: low", "Attack vector: network", "Critical severity", "Has fix"],
        "impactedVersions": ["<1.11.27"],
        "publishedDate": "2019-12-18T20:15:00+01:00",
        "discoveredDate": "2019-12-18T19:15:00Z",
        "fixDate": "2019-12-18T20:15:00+01:00",
    }
    package_1 = {
        "package_registry": "https://registry.npmjs.org/",
        "is_private_registry": False,
        "linesNumbers": [2, 3],
    }
    package_2 = {
        "package_registry": "https://registry.npmjs.org/",
        "is_private_registry": False,
        "linesNumbers": [5, 6],
    }

    record_1 = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        licenses="OSI_BDS",
        package=package_1,
        file_line_range=get_package_lines(package_1),
    )
    record_2 = create_report_cve_record(
        rootless_file_path=rootless_file_path,
        file_abs_path=file_abs_path,
        check_class=check_class,
        vulnerability_details=vulnerability_details,
        licenses="OSI_BDS",
        package=package_2,
        file_line_range=get_package_lines(package_2),
    )

    report = Report(CheckType.SCA_PACKAGE)
    report.add_resource(record_1.resource)
    report.add_record(record_1)
    report.add_resource(record_2.resource)
    report.add_record(record_2)

    # when
    cyclonedx = CycloneDX([report], "repoid/test")

    # then
    assert len(cyclonedx.bom.components) == 1

    component = next(iter(cyclonedx.bom.components))
    assert component.bom_ref.value == "pkg:pypi/repoid/test/requirements.txt/django@1.2"


def test_create_schema_version_1_3(mocker: MockerFixture):
    # given
    test_file = Path(__file__).parent / "fixtures/main.tf"
    repo_id = "acme/example"
    report = Runner().run(root_folder="", files=[str(test_file)])

    mocker.patch.dict(os.environ, {"CHECKOV_CYCLONEDX_SCHEMA_VERSION": "1.3"})

    # when
    cyclonedx = CycloneDX(repo_id=repo_id, reports=[report])
    output = cyclonedx.get_xml_output()

    # then
    assert len(cyclonedx.bom.components) == 1
    assert len(cyclonedx.bom.vulnerabilities) == 6

    assert "http://cyclonedx.org/schema/bom/1.3" in output


def test_create_library_component_maven_package() -> None:
    # given
    cyclone = CycloneDX([Report(CheckType.SCA_IMAGE)], repo_id="12345")
    package = {"name": 'org.bouncycastle_bcpkix-jdk15on',
               "version": '1.69.00'}
    rootless_file_path = "Dockerfile"

    resource = ExtraResource(
        file_abs_path="/path/to/Dockerfile",
        file_path=rootless_file_path,
        resource=f"{rootless_file_path}.{package['name']}",
        vulnerability_details={
            "package_name": package["name"],
            "package_version": package["version"],
            "licenses": "Unknown",
            "package_type": 'jar',
        },
    )

    component = cyclone.create_library_component(resource, CheckType.SCA_IMAGE)

    assert component.purl.name == 'bcpkix-jdk15on'
    assert component.purl.version == '1.69.00'
    assert component.purl.namespace == '12345/Dockerfile/org.bouncycastle'


def test_create_library_component_maven_package_without_group_name() -> None:
    # given
    cyclone = CycloneDX([Report(CheckType.SCA_IMAGE)], repo_id="12345")
    package = {"name": 'bcpkix-jdk15on',
               "version": '1.69.00'}
    rootless_file_path = "Dockerfile"

    resource = ExtraResource(
        file_abs_path="/path/to/Dockerfile",
        file_path=rootless_file_path,
        resource=f"{rootless_file_path}.{package['name']}",
        file_line_range=[2, 5],
        vulnerability_details={
            "package_name": package["name"],
            "package_version": package["version"],
            "licenses": "Unknown",
            "package_type": 'jar'
        },
    )

    component = cyclone.create_library_component(resource, CheckType.SCA_IMAGE)

    assert component.purl.name == 'bcpkix-jdk15on'
    assert component.purl.version == '1.69.00'
    assert component.purl.namespace == '12345/Dockerfile'
    assert component.properties[0].name == 'endLine'
    assert component.properties[0].value == '5'
    assert component.properties[1].name == 'startLine'
    assert component.properties[1].value == '2'

    resource2 = ExtraResource(
        file_abs_path="/path/to/package.json",
        file_path='package.json',
        resource=f"package.json.{package['name']}",
        file_line_range=[0, 0],
        vulnerability_details={
            "package_name": package["name"],
            "package_version": package["version"],
            "licenses": "Unknown",
            "package_type": 'jar'
        },
    )

    component2 = cyclone.create_library_component(resource2, CheckType.SCA_PACKAGE)
    assert component2.properties.key is None


def test_create_json_output():
    # given
    test_file = Path(__file__).parent / "fixtures/main.tf"
    repo_id = "acme/example"
    report = Runner().run(root_folder="", files=[str(test_file)])

    # when
    cyclonedx = CycloneDX(repo_id=repo_id, reports=[report])
    output = json.loads(cyclonedx.get_json_output())

    # then
    assert output["$schema"] == "http://cyclonedx.org/schema/bom-1.4.schema.json"
    assert len(output["components"]) == 1
    assert len(output["dependencies"]) == 1
    assert len(output["vulnerabilities"]) == 6



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.azure.AppGatewayWAFACLCVE202144228 import check
from checkov.terraform.runner import Runner


class TestAppGatewayWAFACLCVE202144228(unittest.TestCase):
    def test(self):
        # given
        test_files_dir = Path(__file__).parent / "example_AppGatewayWAFACLCVE202144228"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "azurerm_web_application_firewall_policy.owasp_3_1_default",
            "azurerm_web_application_firewall_policy.owasp_3_2_default",
            "azurerm_web_application_firewall_policy.version_3_1_default",
            "azurerm_web_application_firewall_policy.owasp_3_1_disabled_different",
            "azurerm_web_application_firewall_policy.empty_disabled_rules"
        }
        failing_resources = {
            "azurerm_web_application_firewall_policy.owasp_3_0",
            "azurerm_web_application_firewall_policy.owasp_3_1_disabled",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], len(passing_resources))
        self.assertEqual(summary["failed"], len(failing_resources))
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)

        # check especially for the evaluated keys
        actual_evaluated_keys = next(
            c.check_result["evaluated_keys"]
            for c in report.failed_checks
            if c.resource == "azurerm_web_application_firewall_policy.owasp_3_1_disabled"
        )
        expected_evaluated_keys = [
            "managed_rules/[0]/managed_rule_set[0]/type",
            "managed_rules/[0]/managed_rule_set[0]/version",
            "managed_rules/[0]/managed_rule_set[0]/rule_group_override/[0]/rule_group_name",
            "managed_rules/[0]/managed_rule_set[0]/rule_group_override/[0]/disabled_rules",
        ]
        self.assertCountEqual(expected_evaluated_keys, actual_evaluated_keys)


if __name__ == "__main__":
    unittest.main()



import unittest

import hcl2
from checkov.terraform.checks.resource.github.WebhookInsecureSsl import check
from checkov.common.models.enums import CheckResult


class TestWebhookInsecureSsl(unittest.TestCase):

    def test_success(self):
        hcl_res = hcl2.loads("""
        resource "github_repository_webhook" "foo" {
        repository = github_repository.repo.name
        name = "web"
        configuration {
            insecure_ssl = false
        }
        }
        """)
        resource_conf = hcl_res['resource'][0]['github_repository_webhook']['foo']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure(self):
        hcl_res = hcl2.loads("""
        resource "github_repository_webhook" "foo" {
        repository = github_repository.repo.name
        name = "web"
        configuration {
            insecure_ssl = true
        }
        }
        """)
        resource_conf = hcl_res['resource'][0]['github_repository_webhook']['foo']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

if __name__ == '__main__':
    unittest.main()



from pathlib import Path
from unittest import mock

import pytest
from pytest_mock import MockerFixture

from checkov.common.output.report import CheckType
from checkov.runner_filter import RunnerFilter
from checkov.terraform.runner import Runner
from tests.common.image_referencer.test_utils import mock_get_empty_license_statuses_async, \
    mock_get_image_cached_result_async
from tests.graph_utils.utils import GRAPH_FRAMEWORKS

RESOURCES_PATH = Path(__file__).parent / "resources/azure"


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_batch_resources(mocker: MockerFixture, graph_framework):
    # given
    file_name = "batch.tf"
    image_name = "centos7"
    code_lines = "1-25"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    tf_report = next(report for report in reports if report.check_type == CheckType.TERRAFORM)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(tf_report.resources) == 1
    assert len(tf_report.passed_checks) == 0
    assert len(tf_report.failed_checks) == 0
    assert len(tf_report.skipped_checks) == 0
    assert len(tf_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go"}
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_containers_resources(mocker: MockerFixture, graph_framework):
    # given
    file_name = "containers.tf"
    image_name_1 = "busybox"
    image_name_2 = "ubuntu:20.04"
    code_lines_1 = "1-37"
    code_lines_2 = "1-37"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    tf_report = next(report for report in reports if report.check_type == CheckType.TERRAFORM)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(tf_report.resources) == 1
    assert len(tf_report.passed_checks) == 2
    assert len(tf_report.failed_checks) == 2
    assert len(tf_report.skipped_checks) == 0
    assert len(tf_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 2
    assert sca_image_report.resources == {
        f"{file_name} ({image_name_1} lines:{code_lines_1} (sha256:2460522297)).go",
        f"{file_name} ({image_name_2} lines:{code_lines_2} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 6
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_app_service_linux_function_resources(mocker: MockerFixture, graph_framework):
    # given
    file_name = "app_service_linux_function.tf"
    image_name_1 = "azure-app-service/samples/aspnethelloworld:latest"
    image_name_2 = "azure-functions/python:4-python3.10-appservice"
    code_lines_1 = "1-18"
    code_lines_2 = "20-35"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    tf_report = next(report for report in reports if report.check_type == CheckType.TERRAFORM)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(tf_report.resources) == 2
    assert len(tf_report.passed_checks) == 4
    assert len(tf_report.failed_checks) == 4
    assert len(tf_report.skipped_checks) == 0
    assert len(tf_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 2
    assert sca_image_report.resources == {
        f"{file_name} ({image_name_1} lines:{code_lines_1} (sha256:2460522297)).go",
        f"{file_name} ({image_name_2} lines:{code_lines_2} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 6
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_app_service_linux_web_resources(mocker: MockerFixture, graph_framework):
    # given
    file_name = "app_service_linux_web.tf"
    image_name_1 = "mcr.microsoft.com/appsvc/staticsite:latest"
    image_name_2 = "busybox:latest"
    code_lines_1 = "1-20"
    code_lines_2 = "23-40"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    tf_report = next(report for report in reports if report.check_type == CheckType.TERRAFORM)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(tf_report.resources) == 2
    assert len(tf_report.passed_checks) == 5
    assert len(tf_report.failed_checks) == 14
    assert len(tf_report.skipped_checks) == 0
    assert len(tf_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 2
    assert sca_image_report.resources == {
        f"{file_name} ({image_name_1} lines:{code_lines_1} (sha256:2460522297)).go",
        f"{file_name} ({image_name_2} lines:{code_lines_2} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 6
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_spring_cloud_resources(mocker: MockerFixture, graph_framework):
    # given
    file_name = "spring_cloud.tf"
    image_name = "springio/gs-spring-boot-docker"
    code_lines = "1-15"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    tf_report = next(report for report in reports if report.check_type == CheckType.TERRAFORM)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(tf_report.resources) == 1
    assert len(tf_report.passed_checks) == 0
    assert len(tf_report.failed_checks) == 0
    assert len(tf_report.skipped_checks) == 0
    assert len(tf_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 1
    assert sca_image_report.resources == {f"{file_name} ({image_name} lines:{code_lines} (sha256:2460522297)).go"}
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 3
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0


@pytest.mark.parametrize("graph_framework", GRAPH_FRAMEWORKS)
def test_app_service_windows_web_resources(mocker: MockerFixture, graph_framework):
    # given
    file_name = "app_service_windows_web.tf"
    image_name_1 = "hello-world:latest"
    image_name_2 = "busybox:latest"
    code_lines_1 = "1-20"
    code_lines_2 = "22-39"
    test_file = RESOURCES_PATH / file_name
    runner_filter = RunnerFilter(run_image_referencer=True)

    mocker.patch(
        "checkov.common.images.image_referencer.image_scanner.get_scan_results_from_cache_async",
        side_effect=mock_get_image_cached_result_async,
    )
    mocker.patch(
        "checkov.common.images.image_referencer.get_license_statuses_async",
        side_effect=mock_get_empty_license_statuses_async,
    )

    # when
    with mock.patch.dict('os.environ', {'CHECKOV_GRAPH_FRAMEWORK': graph_framework}):
        reports = Runner().run(root_folder="", files=[str(test_file)], runner_filter=runner_filter)

    # then
    assert len(reports) == 2

    tf_report = next(report for report in reports if report.check_type == CheckType.TERRAFORM)
    sca_image_report = next(report for report in reports if report.check_type == CheckType.SCA_IMAGE)

    assert len(tf_report.resources) == 2
    assert len(tf_report.passed_checks) == 5
    # Changed from 13 to 14 due to PR #5687
    assert len(tf_report.failed_checks) == 14
    assert len(tf_report.skipped_checks) == 0
    assert len(tf_report.parsing_errors) == 0

    assert len(sca_image_report.resources) == 2
    assert sca_image_report.resources == {
        f"{file_name} ({image_name_1} lines:{code_lines_1} (sha256:2460522297)).go",
        f"{file_name} ({image_name_2} lines:{code_lines_2} (sha256:2460522297)).go",
    }
    assert len(sca_image_report.passed_checks) == 0
    assert len(sca_image_report.failed_checks) == 6
    assert len(sca_image_report.skipped_checks) == 0
    assert len(sca_image_report.parsing_errors) == 0



import unittest
from pathlib import Path

from checkov.runner_filter import RunnerFilter
from checkov.terraform.checks.resource.aws.GithubActionsOIDCTrustPolicyOnRole import check
from checkov.terraform.runner import Runner


class TestGithubActionsOIDCTrustPolicyOnRole(unittest.TestCase):
    """
    Resource-level mirror of CKV_AWS_358 (data-source-only).

    This test asserts behavioral parity with
    tests/terraform/checks/data/aws/test_GithubActionsOIDCTrustPolicy.py
    when the same trust policy is expressed as an inline
    `aws_iam_role.assume_role_policy = jsonencode({...})` instead of a
    `data "aws_iam_policy_document"` block.

    Extra fixture `pass-fm-customer` locks in the XSUP-69131 regression
    marker: Freddie Mac's `repo:freddiemac/*` value is intentionally
    treated as PASSED, matching CKV_AWS_358's `pass-org-only` semantics.
    """

    def test(self):
        # given
        test_files_dir = Path(__file__).parent / "example_GithubActionsOIDCTrustPolicyOnRole"

        # when
        report = Runner().run(root_folder=str(test_files_dir), runner_filter=RunnerFilter(checks=[check.id]))

        # then
        summary = report.get_summary()

        passing_resources = {
            "aws_iam_role.pass1",
            "aws_iam_role.pass2",
            "aws_iam_role.pass3",
            "aws_iam_role.pass_aud_first",
            "aws_iam_role.pass-org-only",
            "aws_iam_role.pass-gh-org",
            "aws_iam_role.pass-fm-customer",
        }
        failing_resources = {
            "aws_iam_role.fail1",
            "aws_iam_role.fail2",
            "aws_iam_role.fail-wildcard",
            "aws_iam_role.fail-abusable",
            "aws_iam_role.fail-wildcard-assertion",
            "aws_iam_role.fail-misused-repo",
        }

        passed_check_resources = {c.resource for c in report.passed_checks}
        failed_check_resources = {c.resource for c in report.failed_checks}

        self.assertEqual(summary["passed"], len(passing_resources))
        self.assertEqual(summary["failed"], len(failing_resources))
        self.assertEqual(summary["skipped"], 0)
        self.assertEqual(summary["parsing_errors"], 0)

        self.assertEqual(passing_resources, passed_check_resources)
        self.assertEqual(failing_resources, failed_check_resources)


if __name__ == "__main__":
    unittest.main()



import os
import shutil
import unittest
import logging
from pathlib import Path
from unittest import mock

from checkov.common.util.consts import DEFAULT_EXTERNAL_MODULES_DIR
from checkov.terraform.module_loading.module_finder import (
    ModuleDownload,
    _download_module,
    find_modules,
    find_tf_managed_modules,
    should_download,
    load_tf_modules
)
from checkov.terraform.module_loading.registry import module_loader_registry


class TestModuleFinder(unittest.TestCase):
    @staticmethod
    def get_src_dir():
        cur_dir = os.path.abspath(os.path.dirname(__file__))
        return os.path.join(cur_dir, 'data', 'tf_module_downloader', 'public_modules')

    def test_module_finder(self):
        modules = find_modules(self.get_src_dir())
        self.assertEqual(2, len(modules), f"modules: {list(map(lambda mod: mod.module_link, modules))}")
        remote_modules = list(filter(lambda mod: should_download(mod.module_link), modules))
        self.assertEqual(1, len(remote_modules))
        for m in remote_modules:
            if 'terraform-aws-modules' in m.module_link:
                self.assertEqual('~> 2.1.0', m.version)
            else:
                self.assertIsNone(m.version)

    def test_module_finder_ignore_comments(self):
        modules = find_modules(self.get_src_dir())
        module_list = list(map(lambda mod: mod.module_link, modules))
        for m in module_list:
            self.assertIn(m, ["terraform-aws-modules/s3-bucket/aws",
                              "../../../../../../../platform/src/stacks/accountStack"])

    def test_module_finder_nested_blocks(self):
        cur_dir = os.path.abspath(os.path.dirname(__file__))
        src_dir = os.path.join(cur_dir, 'data', 'nested_modules')
        modules = find_modules(src_dir)
        self.assertEqual(1, len(modules))
        self.assertEqual("3.14.0", modules[0].version)

    def test_downloader(self):
        modules = find_modules(self.get_src_dir())

        remote_modules = [m for m in modules if should_download(m.module_link)]
        module_loader_registry.download_external_modules = True
        load_tf_modules(os.path.join(self.get_src_dir()), run_parallel=True)
        downloaded_modules = os.listdir(os.path.join(self.get_src_dir(), DEFAULT_EXTERNAL_MODULES_DIR))
        distinct_roots = {md.module_link.split('/')[0] for md in remote_modules}
        shutil.rmtree(os.path.join(self.get_src_dir(), DEFAULT_EXTERNAL_MODULES_DIR))
        self.assertEqual(len(downloaded_modules), 1)
        self.assertEqual(len(distinct_roots), 1)


def test_dem_warning(caplog):
    """
    Test that the --download-external-modules flag warning message is only
    logged if the flag is not specified on the command line, and that
    module download warnings are not logged if the flag is set to False.
    """
    caplog.set_level(logging.WARNING)
    module_loader_registry.download_external_modules = None
    _download_module(module_loader_registry, ModuleDownload('xxx'))
    assert 'Failed to download module' in caplog.text
    assert '--download-external-modules flag' in caplog.text
    caplog.clear()

    module_loader_registry.download_external_modules = True
    _download_module(module_loader_registry, ModuleDownload('xxx'))
    assert 'Failed to download module' in caplog.text
    assert '--download-external-modules flag' not in caplog.text
    caplog.clear()

    module_loader_registry.download_external_modules = False
    _download_module(module_loader_registry, ModuleDownload('xxx'))
    assert 'Failed to download module' not in caplog.text
    assert '--download-external-modules flag' not in caplog.text

def test_tf_managed_and_comment_out_modules():
    src_path = Path(__file__).parent / 'data' / 'tf_managed_modules'
    modules = find_tf_managed_modules(str(src_path))

    assert len(modules) == 1
    assert modules[0].tf_managed is True
    assert modules[0].address == "registry.terraform.io/terraform-aws-modules/cloudwatch/aws//modules/log-group:4.1.0"
    assert modules[0].module_link == ".terraform/modules/log_group/modules/log-group"

def test_tf_managed_submodules():
    modules = find_tf_managed_modules(Path(__file__).parent / 'data' / 'tf_managed_submodules')
    assert len(modules) == 2
    assert modules[0].tf_managed is True
    assert modules[0].address == 'somewhere/a:0'
    assert modules[0].module_name == 'a'
    assert modules[0].module_link == '.terraform/modules/a'
    assert modules[1].tf_managed is True
    assert modules[1].address == 'somewhere/b:1'
    assert modules[1].module_name == 'a.b'
    assert modules[1].module_link == '.terraform/modules/a.b'



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.RDSClusterSnapshotEncrypted import check
from checkov.common.models.enums import CheckResult


class TestRDSClusterSnapshotEncrypted(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
            resource "aws_db_cluster_snapshot" "example" {
              db_cluster_identifier          = aws_rds_cluster.example.id
              db_cluster_snapshot_identifier = "resourcetestsnapshot1234"
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_db_cluster_snapshot']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_failure2(self):
        hcl_res = hcl2.loads("""
            resource "aws_db_cluster_snapshot" "example" {
              db_cluster_identifier          = aws_rds_cluster.example.id
              db_cluster_snapshot_identifier = "resourcetestsnapshot1234"
              storage_encrypted = false
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_db_cluster_snapshot']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success2(self):
        hcl_res = hcl2.loads("""
            resource "aws_db_cluster_snapshot" "example" {
              db_cluster_identifier          = aws_rds_cluster.example.id
              db_cluster_snapshot_identifier = "resourcetestsnapshot1234"
              storage_encrypted = true
            }
        """)
        resource_conf = hcl_res['resource'][0]['aws_db_cluster_snapshot']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import pytest

from checkov.azure_pipelines.runner import Runner

@pytest.mark.parametrize(
    "key,file_path,expected_key, start_line, end_line",
    [
        ('jobs.jobs.CKV_AZUREPIPELINES_1[32:39]', '/checkov/tests/azure_pipelines/resources/azure-pipelines.yml',
         'jobs[0](FailTag)', 32, 39),
        ('stages[].jobs[].stages[].jobs[].CKV_AZUREPIPELINES_1[14:22]', '/checkov/tests/azure_pipelines/resources/azure-pipelines.yml',
         'stages[0](Example).jobs[0](FailNoTagDisplayName)', 14, 22),
        ('stages[].jobs[].stages[].jobs[].CKV_AZUREPIPELINES_1[22:29]', '/checkov/tests/azure_pipelines/resources/azure-pipelines.yml',
         'stages[0](Example).jobs[1](PassDigest)', 22, 29)
    ],
)
def test_get_resource(key, file_path, expected_key, definitions, supported_entities, start_line, end_line):
    runner = Runner()
    runner.definitions = definitions
    new_key = runner.get_resource(file_path, key, [], start_line, end_line)

    assert new_key == expected_key


import unittest

import os

from checkov.common.bridgecrew.check_type import CheckType
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration
from checkov.common.bridgecrew.severities import BcSeverities, Severities
from checkov.secrets.runner import Runner
from checkov.runner_filter import RunnerFilter


class TestRunnerValid(unittest.TestCase):

    def setUp(self) -> None:
        self.orig_metadata = metadata_integration.check_metadata

    def test_runner_failing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/cfn"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets']))
        self.assertEqual(len(report.failed_checks), 2)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_sanity_check_secrets(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/sanity/secrets"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], checks=['CKV_SECRET_6']))
        self.assertEqual(len(report.failed_checks), 6)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_fp_sanity_check_secrets_non_iac(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/sanity/iac_fp"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], checks=['CKV_SECRET_6'], enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_fp_sanity_check_secrets_iac(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/sanity/non_iac_fp"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], checks=['CKV_SECRET_6'], enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_sanity_check_non_secrets(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/sanity/non_secrets"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], checks=['CKV_SECRET_6']))
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_runner_honors_enforcement_rules(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/cfn"
        runner = Runner()
        filter = RunnerFilter(framework=['secrets'], use_enforcement_rules=True)
        # this is not quite a true test, because the checks don't have severities. However, this shows that the check registry
        # passes the report type properly to RunnerFilter.should_run_check, and we have tests for that method
        filter.enforcement_rule_configs = {CheckType.SECRETS: Severities[BcSeverities.OFF]}
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=filter)
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.parsing_errors), 0)
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 0)
        report.print_console()

    def test_runner_passing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/terraform"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets']))
        self.assertEqual(len(report.passed_checks), 0)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.failed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_runner_tf_failing_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/terraform_failed"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets']))
        self.assertEqual(2, len(report.failed_checks))
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(report.skipped_checks, [])
        report.print_console()

    def test_runner_tf_skip_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/terraform_skip"

        report = Runner().run(
            root_folder=valid_dir_path,
            external_checks_dir=None,
            runner_filter=RunnerFilter(framework=['secrets'])
        )

        self.assertEqual(len(report.skipped_checks), 1)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(len(report.skipped_checks), 1)

        report.print_console()

    def test_runner_specific_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/cfn"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], checks=['CKV_SECRET_2']))
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])

    def test_runner_wildcard_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/cfn"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], checks=['CKV_SECRET*']))
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.failed_checks), 2)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])

    def test_runner_skip_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/cfn"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], skip_checks=['CKV_SECRET_2']))
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.failed_checks[0].check_id, 'CKV_SECRET_6')
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])

    def test_record_has_severity(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/cfn"

        metadata_integration.check_metadata = {
            'CKV_SECRET_2': {
                'severity': Severities[BcSeverities.LOW]
            }
        }

        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], checks=['CKV_SECRET_2']))
        self.assertEqual(report.failed_checks[0].severity, Severities[BcSeverities.LOW])

    def test_runner_check_severity(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/cfn"

        metadata_integration.check_metadata = {
            'CKV_SECRET_2': {
                'severity': Severities[BcSeverities.LOW]
            },
            'CKV_SECRET_6': {
                'severity': Severities[BcSeverities.HIGH]
            }
        }

        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], checks=['MEDIUM']))
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.failed_checks[0].check_id, 'CKV_SECRET_6')
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])

    def test_runner_skip_check_severity(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/cfn"

        metadata_integration.check_metadata = {
            'CKV_SECRET_2': {
                'severity': Severities[BcSeverities.LOW]
            },
            'CKV_SECRET_6': {
                'severity': Severities[BcSeverities.HIGH]
            }
        }

        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], skip_checks=['MEDIUM']))
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.failed_checks), 1)
        self.assertEqual(report.failed_checks[0].check_id, 'CKV_SECRET_6')
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])

    def test_runner_skip_check_wildcard(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/cfn"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], skip_checks=['CKV_SECRET*']))
        self.assertEqual(len(report.skipped_checks), 0)
        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])

    def test_runner_multiple_files(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets']))
        self.assertEqual(9, len(report.failed_checks))
        self.assertEqual(report.parsing_errors, [])
        self.assertEqual(report.passed_checks, [])
        self.assertEqual(len(report.skipped_checks), 1)

    def test_runner_bc_ids(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources"
        runner = Runner()
        # the other tests will implicitly test this value being None

        metadata_integration.check_metadata = {
            'CKV_SECRET_2': {
                'id': 'BC_GIT_2'
            }
        }

        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets']))
        for fc in report.failed_checks:
            if fc.check_id == 'CKV_SECRET_2':
                self.assertEqual(fc.bc_check_id, 'BC_GIT_2')
            else:
                self.assertIsNone(fc.bc_check_id)

    def tearDown(self) -> None:
        metadata_integration.check_metadata = self.orig_metadata

    def test_runner_requested_file_type_only_ts(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path,
                            external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'],
                                                       block_list_secret_scan=['.py', 'Dockerfile', '.tf', '.yml'],
                                                       enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 2)

    def test_runner_requested_file_type_only_py(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], block_list_secret_scan=['.ts', 'Dockerfile', '.tf', '.yml'],
                                                       enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 2)

    def test_runner_requested_file_type_only_yml(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], block_list_secret_scan=['.py', 'Dockerfile', '.tf', '.ts'],
                                                       enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 2)

    def test_runner_requested_file_type_only_tf(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'],
                                                       block_list_secret_scan=['.py', 'Dockerfile', '.ts', '.yml'],
                                                       enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 3)

    def test_runner_requested_file_type_only_tf_yml(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], block_list_secret_scan=['.py', 'Dockerfile', '.ts'],
                                                       enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 5)

    def test_runner_requested_file_type_all(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 13)

    def test_runner_requested_file_only_dockerfile(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], block_list_secret_scan=['.py', '.tf', '.ts', '.yml'],
                                                       enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 4)


    def test_runner_no_requested_file(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets']))
        self.assertEqual(len(report.failed_checks), 9)

    def test_true_positive_py(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_file_path = current_dir + "/resources/file_type/test.py"
        runner = Runner()
        report = runner.run(root_folder=None, files=[valid_file_path], external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 2)

    def test_no_false_positive_yml_2(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/resources/cfn"
        valid_file_path = valid_dir_path + "/secret-no-false-positive.yml"
        runner = Runner()
        report = runner.run(root_folder=None, files=[valid_file_path], external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'],
                                                       enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 0)

    def test_runner_entropy_source_files(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/test_entropy_source_files"
        runner = Runner()
        report = runner.run(root_folder=valid_dir_path, runner_filter=RunnerFilter(framework=['secrets'],
                                                                                   enable_secret_scan_all_files=True))
        self.assertEqual(len(report.failed_checks), 2)
        for failed in report.failed_checks:
            if failed.check_id == 'CKV_SECRET_6':
                self.assertEqual(failed.file_line_range, [4, 5])
            elif failed.check_id == 'CKV_SECRET_4':
                self.assertEqual(failed.file_line_range, [6, 7])
            else:
                self.fail(f'Got a bad result: {failed}')

    def test_runner_omit_multiple_secrets_in_line(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        valid_dir_path = current_dir + "/omit_multiple_secrets/test"
        runner = Runner()
        runner_filter = RunnerFilter(framework=['secrets'], enable_secret_scan_all_files=True)
        report = runner.run(root_folder=valid_dir_path, runner_filter=runner_filter)
        self.assertEqual(len(report.failed_checks), 2)
        assert report.failed_checks[0].code_block[0][1] == "export AWS_ACCESS_KEY_ID=AKIAI**********\\nexport CIRCLE='rk_liv**********'\n"
        assert report.failed_checks[1].code_block[0][1] == "export AWS_ACCESS_KEY_ID=AKIAI**********\\nexport CIRCLE='rk_liv**********'\n"

    def test_metadata_suppression_object_rooted_json_skips_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        file_path = os.path.join(current_dir, "suppressions/metadata_suppression_object.json")

        runner = Runner()
        report = runner.run(root_folder=None, files=[file_path], external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], enable_secret_scan_all_files=True))

        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 2)

    def test_metadata_suppression_array_rooted_json_skips_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        file_path = os.path.join(current_dir, "suppressions/metadata_suppression_array.json")

        runner = Runner()
        report = runner.run(root_folder=None, files=[file_path], external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], enable_secret_scan_all_files=True))

        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 2)

    def test_metadata_suppression_yaml_skips_check(self):
        current_dir = os.path.dirname(os.path.realpath(__file__))
        file_path = os.path.join(current_dir, "suppressions/metadata_suppression.yaml")

        runner = Runner()
        report = runner.run(root_folder=None, files=[file_path], external_checks_dir=None,
                            runner_filter=RunnerFilter(framework=['secrets'], enable_secret_scan_all_files=True))

        self.assertEqual(len(report.failed_checks), 0)
        self.assertEqual(len(report.skipped_checks), 2)

if __name__ == '__main__':
    unittest.main()



import os
import unittest
from pathlib import Path
from checkov.common.util.data_structures_utils import pickle_deepcopy

from checkov.cloudformation.cfn_utils import get_folder_definitions, build_definitions_context, enrich_resources_with_globals
from checkov.common.bridgecrew.integration_features.features.policy_metadata_integration import integration as metadata_integration
from checkov.common.bridgecrew.platform_integration import bc_integration, BcPlatformIntegration
from checkov.common.parsers.node import DictNode
from checkov.cloudformation.parser import TemplateSections

TEST_DIRNAME = os.path.dirname(os.path.realpath(__file__))
RELATIVE_PATH = "file_formats"


class TestCfnUtils(unittest.TestCase):
    def setUp(self):
        # self.test_root_dir = os.path.realpath(os.path.join(TEST_DIRNAME, RELATIVE_PATH))
        self.test_root_dir = Path(TEST_DIRNAME) / RELATIVE_PATH
        integration = BcPlatformIntegration()
        metadata_integration.bc_integration = integration
        integration.get_public_run_config()
        metadata_integration.pre_scan()
        definitions, definitions_raw = get_folder_definitions(str(self.test_root_dir), None)
        self.definitions_context = build_definitions_context(definitions, definitions_raw)

    def tearDown(self) -> None:
        metadata_integration.bc_integration = bc_integration

    def validate_definition_lines(self, definition: DictNode, start_line, end_line, code_lines):
        self.assertEqual(definition["start_line"], start_line)
        self.assertEqual(definition["end_line"], end_line)
        self.assertEqual(len(definition["code_lines"]), code_lines)

    def test_parameters_value(self):
        # Asserting test.yaml file
        yaml_parameters = self.definitions_context[str(self.test_root_dir / "test.yaml")][
            TemplateSections.PARAMETERS.value
        ]
        self.assertIsNotNone(yaml_parameters)
        self.assertEqual(len(yaml_parameters), 2)
        self.validate_definition_lines(yaml_parameters["KmsMasterKeyId"], 4, 7, 4)
        self.validate_definition_lines(yaml_parameters["DBName"], 8, 11, 4)
        # Asserting test2.yaml file
        yaml2_parameters = self.definitions_context[str(self.test_root_dir / "test2.yaml")][
            TemplateSections.PARAMETERS.value
        ]
        self.assertIsNotNone(yaml2_parameters)
        self.assertEqual(len(yaml2_parameters), 1)
        self.validate_definition_lines(yaml2_parameters["LatestAmiId"], 4, 6, 3)
        # Asserting json file
        json_parameters = self.definitions_context[str(self.test_root_dir / "test.json")][
            TemplateSections.PARAMETERS.value
        ]
        self.assertIsNotNone(json_parameters)
        self.assertEqual(len(json_parameters), 2)
        self.validate_definition_lines(json_parameters["KmsMasterKeyId"], 5, 9, 5)
        self.validate_definition_lines(json_parameters["DBName"], 10, 14, 5)

    def test_resources_value(self):
        # Asserting test.yaml file
        yaml_resources = self.definitions_context[str(self.test_root_dir / "test.yaml")][
            TemplateSections.RESOURCES.value
        ]
        self.assertIsNotNone(yaml_resources)
        self.assertEqual(len(yaml_resources), 2)
        self.validate_definition_lines(yaml_resources["MySourceQueue"], 13, 16, 4)
        self.validate_definition_lines(yaml_resources["MyDB"], 17, 37, 21)
        # Asserting test2.yaml file
        yaml2_resources = self.definitions_context[str(self.test_root_dir / "test2.yaml")][
            TemplateSections.RESOURCES.value
        ]
        self.assertIsNotNone(yaml2_resources)
        self.assertEqual(len(yaml2_resources), 4)
        self.validate_definition_lines(yaml2_resources["WebHostStorage"], 12, 23, 12)
        self.validate_definition_lines(yaml2_resources["LogsKey"], 29, 44, 16)
        self.validate_definition_lines(yaml2_resources["LogsKeyAlias"], 46, 50, 5)
        self.validate_definition_lines(yaml2_resources["DBAppInstance"], 52, 184, 133)
        # Asserting json file
        json_resources = self.definitions_context[str(self.test_root_dir / "test.json")][
            TemplateSections.RESOURCES.value
        ]
        self.assertIsNotNone(json_resources)
        self.assertEqual(len(json_resources), 2)
        self.validate_definition_lines(json_resources["MySourceQueue"], 17, 22, 6)
        self.validate_definition_lines(json_resources["MyDB"], 23, 32, 10)

    def test_outputs_value(self):
        # Asserting test.yaml file
        yaml_outputs = self.definitions_context[str(self.test_root_dir / "test.yaml")][TemplateSections.OUTPUTS.value]
        self.assertIsNotNone(yaml_outputs)
        self.assertEqual(len(yaml_outputs), 1)
        self.validate_definition_lines(yaml_outputs["DBAppPublicDNS"], 40, 42, 3)
        # Asserting test2.yaml file
        yaml2_outputs = self.definitions_context[str(self.test_root_dir / "test2.yaml")][TemplateSections.OUTPUTS.value]
        self.assertIsNotNone(yaml2_outputs)
        self.assertEqual(len(yaml2_outputs), 5)
        self.validate_definition_lines(yaml2_outputs["EC2PublicDNS"], 187, 191, 5)
        self.validate_definition_lines(yaml2_outputs["VpcId"], 192, 195, 4)
        self.validate_definition_lines(yaml2_outputs["PublicSubnet"], 196, 198, 3)
        self.validate_definition_lines(yaml2_outputs["PublicSubnet2"], 200, 202, 3)
        self.validate_definition_lines(yaml2_outputs["UserName"], 204, 206, 3)
        # Asserting json file
        json_outputs = self.definitions_context[str(self.test_root_dir / "test.json")][TemplateSections.OUTPUTS.value]
        self.assertIsNotNone(json_outputs)
        self.assertEqual(len(json_outputs), 1)
        self.validate_definition_lines(json_outputs["DBAppPublicDNS"], 35, 38, 4)

    def test_skipped_check_exists(self):
        skipped_checks = self.definitions_context[str(self.test_root_dir / "test.yaml")][
            TemplateSections.RESOURCES.value
        ]["MyDB"]["skipped_checks"]
        self.assertEqual(len(skipped_checks), 3)
        self.assertCountEqual(
            skipped_checks,
            [
                {
                    "id": "CKV_AWS_16",
                    "suppress_comment": "Ensure all data stored in the RDS is securely encrypted at rest",
                    "bc_id": "BC_AWS_GENERAL_4",
                    "line_number": 30
                },
                {
                    "id": "CKV_AWS_17",
                    "suppress_comment": "Ensure all data stored in RDS is not publicly accessible",
                    "bc_id": "BC_AWS_PUBLIC_2",
                },
                {
                    "id": "CKV_AWS_157",
                    "suppress_comment": "Ensure that RDS instances have Multi-AZ enabled",
                    "bc_id": "BC_AWS_GENERAL_73",
                },
            ],
        )

    def test_globals_absent(self):
        original_template = {'Resources': {}}
        enriched_template = enrich_resources_with_globals(original_template)
        self.assertEqual(enriched_template, original_template)

    def test_globals_no_resources(self):
        original_template = {'Globals': {}, 'Resources': {}}
        enriched_template = enrich_resources_with_globals(original_template)
        self.assertEqual(enriched_template, original_template)

    def test_globals_applicable(self):
        start_mark = object()  # Placeholder for a real starting position in a file
        end_mark = object()  # Placeholder for a real ending position in a file

        # Setting up original template with DictNode
        original_globals = DictNode({'Function': {'Timeout': 30}}, start_mark, end_mark)
        original_properties = DictNode({'MemorySize': 128}, start_mark, end_mark)
        original_resources = DictNode({
            'MyFunction': {
                'Type': 'AWS::Serverless::Function',
                'Properties': original_properties
            }
        }, start_mark, end_mark)
        original_template = DictNode({
            'Globals': original_globals,
            'Resources': original_resources
        }, start_mark, end_mark)

        # Setting up expected template with DictNode
        expected_properties = pickle_deepcopy(original_properties)
        expected_properties['Timeout'] = 30
        expected_resources = DictNode({
            'MyFunction': {
                'Type': 'AWS::Serverless::Function',
                'Properties': expected_properties
            }
        }, start_mark, end_mark)
        expected_template = DictNode({
            'Globals': original_globals,
            'Resources': expected_resources
        }, start_mark, end_mark)

        # Performing the enrichment
        enriched_template = enrich_resources_with_globals(original_template)
        self.assertEqual(enriched_template, expected_template)

    def test_deep_merge_non_conflicting(self):
        # Example marks, in real cases these would be meaningful values
        start_mark = object()
        end_mark = object()

        dict1 = DictNode({'a': 1}, start_mark, end_mark)
        dict2 = DictNode({'b': 2}, start_mark, end_mark)
        merged_result = DictNode.deep_merge(dict1, dict2)

        self.assertEqual(merged_result, DictNode({'a': 1, 'b': 2}, start_mark, end_mark))

    def test_deep_merge_overlapping_scalars(self):
        start_mark = object()
        end_mark = object()

        dict1 = DictNode({'a': 1}, start_mark, end_mark)
        dict2 = DictNode({'a': 2}, start_mark, end_mark)
        merged_result = DictNode.deep_merge(dict1, dict2)

        self.assertEqual(merged_result, DictNode({'a': 1}, start_mark, end_mark))

    def test_deep_merge_recursive_dict_nodes(self):
        start_mark = object()
        end_mark = object()

        dict1_inner = DictNode({'c': 3}, start_mark, end_mark)
        dict1 = DictNode({'a': dict1_inner}, start_mark, end_mark)

        dict2_inner = DictNode({'d': 4}, start_mark, end_mark)
        dict2 = DictNode({'a': dict2_inner}, start_mark, end_mark)

        merged_result = DictNode.deep_merge(dict1, dict2)
        expected_result = DictNode({'a': DictNode({'c': 3, 'd': 4}, start_mark, end_mark)}, start_mark, end_mark)

        self.assertEqual(merged_result, expected_result)


if __name__ == "__main__":
    unittest.main()



from aws_cdk import core
from aws_cdk import aws_cloudfront as cloudfront

class MyCloudFrontDistributionStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define CloudFront Distribution with ViewerProtocolPolicy set to allow_all
        distribution = cloudfront.CfnDistribution(
            self, 'MyCloudFrontDistribution',
            distribution_config=cloudfront.CfnDistribution.DistributionConfigProperty(
                default_cache_behavior=cloudfront.CfnDistribution.DefaultCacheBehaviorProperty(
                    viewer_protocol_policy='abc'
                ),
                # Add other properties for the distribution config as needed
            )
        )

app = core.App()
MyCloudFrontDistributionStack(app, "MyCloudFrontDistributionStack")
app.synth()

class MyCloudFrontDistributionStack2(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define CloudFront Distribution with CacheBehavior and ViewerProtocolPolicy
        distribution = cloudfront.CfnDistribution(
            self, 'MyCloudFrontDistribution',
            distribution_config=cloudfront.CfnDistribution.DistributionConfigProperty(
                cache_behaviors=[
                    cloudfront.CfnDistribution.CacheBehaviorProperty(
                        path_pattern='/path-to-cache',
                        target_origin_id='my-target-origin-id',
                    )
                ],
                # Other distribution configuration properties
            )
        )

app = core.App()
MyCloudFrontDistributionStack2(app, "MyCloudFrontDistributionStack2")
app.synth()



import pytest

from checkov.policies_3d.syntax.cves_syntax import RiskFactorCVEContains


@pytest.fixture
def cve_report_string_risk_factors():
    return {
        'cveId': 'cveId',
        'status': 'status',
        'severity': 'severity',
        'packageName': 'packageName',
        'packageVersion': 'packageVersion',
        'link': 'link',
        'publishedDate': 'publishedDate',
        'cvss': 'cvss',
        'vector': 'vector',
        'description': 'description',
        'riskFactors': 'DoS'
    }

@pytest.fixture
def cve_report_list_risk_factors():
    return {
        'cveId': 'cveId',
        'status': 'status',
        'severity': 'severity',
        'packageName': 'packageName',
        'packageVersion': 'packageVersion',
        'link': 'link',
        'publishedDate': 'publishedDate',
        'cvss': 'cvss',
        'vector': 'vector',
        'description': 'description',
        'riskFactors': ['DoS', 'RCE']
    }

@pytest.fixture
def cve_report_list_prefix_risk_factors():
    return {
        'cveId': 'cveId',
        'status': 'status',
        'severity': 'severity',
        'packageName': 'packageName',
        'packageVersion': 'packageVersion',
        'link': 'link',
        'publishedDate': 'publishedDate',
        'cvss': 'cvss',
        'vector': 'vector',
        'description': 'description',
        'riskFactors': ['DoS - High', 'RCE']
    }


def test_risk_factor_cve_contains_normalizes_risk_factors(cve_report_string_risk_factors):
    # Arrange
    risk_factors = ['dos']
    predicate = RiskFactorCVEContains(risk_factors, cve_report_string_risk_factors)

    # Assert
    assert predicate.risk_factors == ['dos']
    assert predicate.cve_report['riskFactors'] == ['dos']


def test_risk_factor_cve_contains_true(cve_report_string_risk_factors):
    # Arrange
    risk_factors = ['dos']
    predicate = RiskFactorCVEContains(risk_factors, cve_report_string_risk_factors)

    # Act
    predicate()

    # Assert
    assert predicate.is_true

def test_risk_factor_cve_contains_false(cve_report_string_risk_factors):
    # Arrange
    risk_factors = ['not a risk factor']
    predicate = RiskFactorCVEContains(risk_factors, cve_report_string_risk_factors)

    # Act
    predicate()

    # Arrange
    assert not predicate.is_true

def test_risk_factor_cve_contains_true_2(cve_report_list_risk_factors):
    # Arrange
    risk_factors = ['dos']
    predicate = RiskFactorCVEContains(risk_factors, cve_report_list_risk_factors)

    # Act
    predicate()

    # Assert
    assert predicate.is_true

def test_risk_factor_cve_contains_true_3(cve_report_list_prefix_risk_factors):
    # Arrange
    risk_factors = ['dos']
    predicate = RiskFactorCVEContains(risk_factors, cve_report_list_prefix_risk_factors)

    # Act
    predicate()

    # Assert
    assert predicate.is_true


def test_risk_factor_cve_contains_false_2(cve_report_list_risk_factors):
    # Arrange
    risk_factors = ['not a risk factor']
    predicate = RiskFactorCVEContains(risk_factors, cve_report_list_risk_factors)

    # Act
    predicate()

    # Arrange
    assert not predicate.is_true


def test_risk_factor_cve_contains_true_equality(cve_report_list_risk_factors):
    # Arrange
    risk_factors = ['Dos']
    p1 = RiskFactorCVEContains(risk_factors, cve_report_list_risk_factors)
    p2 = RiskFactorCVEContains(risk_factors, cve_report_list_risk_factors)

    # Assert
    assert p1 == p2


def test_risk_factor_cve_contains_false_equality(cve_report_list_risk_factors):
    # Arrange
    risk_factors_1 = ['Dos']
    risk_factors_2 = ['RCE']
    p1 = RiskFactorCVEContains(risk_factors_1, cve_report_list_risk_factors)
    p2 = RiskFactorCVEContains(risk_factors_2, cve_report_list_risk_factors)

    # Assert
    assert p1 != p2



from pathlib import Path

from checkov.ansible.runner import Runner
from checkov.ansible.utils import build_definitions_context, create_definitions

EXAMPLES_DIR = Path(__file__).parent / "examples"


def test_build_definitions_context():
    # given
    file_path = str(EXAMPLES_DIR / "skip.yml")
    definitions, definitions_raw = {}, {}
    definitions[file_path], definitions_raw[file_path] = Runner()._parse_file(f=file_path)

    # when
    context = build_definitions_context(definitions=definitions, definitions_raw=definitions_raw)

    assert len(context) == 1

    definition_context = next(iter(context.values()))
    assert len(definition_context) == 3

    task_context = definition_context["tasks.uri.http"]

    assert task_context["start_line"] == 24
    assert task_context["end_line"] == 31
    assert task_context["code_lines"] == [
        (24, "    - name: http\n"),
        (25, "      #checkov:skip=CKV2_ANSIBLE_1\n"),
        (26, "      uri:\n"),
        (27, "        url: http://www.example.com\n"),
        (28, "        return_content: yes\n"),
        (29, "      register: this\n"),
        (30, "      failed_when: \"'AWESOME' not in this.content\"\n"),
        (31, "\n"),
    ]

    for skip in task_context["skipped_checks"]:
        skip.pop("bc_id", None)  # depending on the test order they are set or not

    assert task_context["skipped_checks"] == [
        {
            "id": "CKV2_ANSIBLE_1",
            "line_number": 25,
            "suppress_comment": "No comment provided",
        }
    ]


def test_create_definitions():
    definitions, definitions_raw = create_definitions(root_folder=str(EXAMPLES_DIR))

    assert len(definitions) > 0
    assert all(key.endswith(".yml") or key.endswith(".yaml") and len(value) > 0 for key, value in definitions.items())


from copy import deepcopy
from unittest import TestCase

from checkov.common.graph.graph_builder.graph_components.block_types import BlockType
from checkov.kubernetes.graph_builder.graph_components.blocks import KubernetesBlock


class TestGraph(TestCase):
    def assert_vertex(self, resource_vertex, resource):
        resource_name = f'{resource["kind"]}.{resource["metadata"].get("namespace", "default")}.{resource["metadata"]["name"]}'
        config = deepcopy(resource)
        attributes = deepcopy(config)
        enrich_attributes(attributes, resource)

        self.assertEqual(resource_name, resource_vertex.name)
        self.assertEqual(resource_name, resource_vertex.id)
        self.assertEqual(BlockType.RESOURCE, resource_vertex.block_type)
        self.assertEqual("Kubernetes", resource_vertex.source)
        self.assertDictEqual(config, resource_vertex.config)
        self.assertDictEqual(attributes, resource_vertex.attributes)


def extract_inner_attributes(attributes):
    attributes_to_add = {}
    for attribute_key in attributes:
        attribute_value = attributes[attribute_key]
        if isinstance(attribute_value, dict) or (
                isinstance(attribute_value, list) and len(attribute_value) > 0 and isinstance(attribute_value[0],
                                                                                              dict)):
            inner_attributes = KubernetesBlock.get_inner_attributes(attribute_key, attribute_value)
            attributes_to_add.update(inner_attributes)
    return attributes_to_add


def enrich_attributes(attributes, resource):
    attributes["resource_type"] = resource["kind"]
    attributes["__startline__"] = resource["__startline__"]
    attributes["__endline__"] = resource["__endline__"]
    attributes.update(extract_inner_attributes(attributes))



from typing import Any, List

from checkov.common.models.enums import CheckCategories
from checkov.terraform.checks.resource.base_resource_negative_value_check import \
    BaseResourceNegativeValueCheck


class CVMDisableMonitorService(BaseResourceNegativeValueCheck):
    def __init__(self):
        name = "Ensure Tencent Cloud CVM monitor service is enabled"
        id = "CKV_TC_3"
        supported_resources = ['tencentcloud_instance']
        categories = [CheckCategories.LOGGING]
        super().__init__(name=name, id=id, categories=categories, supported_resources=supported_resources)

    def get_inspected_key(self) -> str:
        return 'disable_monitor_service/[0]'

    def get_forbidden_values(self) -> List[Any]:
        return [True]


check = CVMDisableMonitorService()



import unittest

import hcl2

from checkov.terraform.checks.resource.azure.IoTNoPublicNetworkAccess import check
from checkov.common.models.enums import CheckResult


class TestIoTNoPublicNetworkAccess(unittest.TestCase):

    def test_success_missing_attribute(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_iothub" "example" {
              name                = "Example-IoTHub"
              resource_group_name = azurerm_resource_group.example.name
              location            = azurerm_resource_group.example.location
            
              sku {
                name     = "S1"
                capacity = "1"
              }
            
              endpoint {
                type                       = "AzureIotHub.StorageContainer"
                connection_string          = azurerm_storage_account.example.primary_blob_connection_string
                name                       = "export"
                batch_frequency_in_seconds = 60
                max_chunk_size_in_bytes    = 10485760
                container_name             = azurerm_storage_container.example.name
                encoding                   = "Avro"
                file_name_format           = "{iothub}/{partition}_{YYYY}_{MM}_{DD}_{HH}_{mm}"
              }
            
              endpoint {
                type              = "AzureIotHub.EventHub"
                connection_string = azurerm_eventhub_authorization_rule.example.primary_connection_string
                name              = "export2"
              }
            
              route {
                name           = "export"
                source         = "DeviceMessages"
                condition      = "true"
                endpoint_names = ["export"]
                enabled        = true
              }
            
              route {
                name           = "export2"
                source         = "DeviceMessages"
                condition      = "true"
                endpoint_names = ["export2"]
                enabled        = true
              }
            
              enrichment {
                key            = "tenant"
                value          = "$twin.tags.Tenant"
                endpoint_names = ["export", "export2"]
              }
            
              tags = {
                purpose = "testing"
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_iothub']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

    def test_failure(self):
        hcl_res = hcl2.loads("""
           resource "azurerm_iothub" "example" {
              name                = "Example-IoTHub"
              resource_group_name = azurerm_resource_group.example.name
              location            = azurerm_resource_group.example.location
            
              sku {
                name     = "S1"
                capacity = "1"
              }
            
              endpoint {
                type                       = "AzureIotHub.StorageContainer"
                connection_string          = azurerm_storage_account.example.primary_blob_connection_string
                name                       = "export"
                batch_frequency_in_seconds = 60
                max_chunk_size_in_bytes    = 10485760
                container_name             = azurerm_storage_container.example.name
                encoding                   = "Avro"
                file_name_format           = "{iothub}/{partition}_{YYYY}_{MM}_{DD}_{HH}_{mm}"
              }
            
              endpoint {
                type              = "AzureIotHub.EventHub"
                connection_string = azurerm_eventhub_authorization_rule.example.primary_connection_string
                name              = "export2"
              }
              public_network_access_enabled = true
              route {
                name           = "export"
                source         = "DeviceMessages"
                condition      = "true"
                endpoint_names = ["export"]
                enabled        = true
              }
            
              route {
                name           = "export2"
                source         = "DeviceMessages"
                condition      = "true"
                endpoint_names = ["export2"]
                enabled        = true
              }
            
              enrichment {
                key            = "tenant"
                value          = "$twin.tags.Tenant"
                endpoint_names = ["export", "export2"]
              }
            
              tags = {
                purpose = "testing"
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_iothub']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
            resource "azurerm_iothub" "example" {
              name                = "Example-IoTHub"
              resource_group_name = azurerm_resource_group.example.name
              location            = azurerm_resource_group.example.location
            
              sku {
                name     = "S1"
                capacity = "1"
              }
            
              endpoint {
                type                       = "AzureIotHub.StorageContainer"
                connection_string          = azurerm_storage_account.example.primary_blob_connection_string
                name                       = "export"
                batch_frequency_in_seconds = 60
                max_chunk_size_in_bytes    = 10485760
                container_name             = azurerm_storage_container.example.name
                encoding                   = "Avro"
                file_name_format           = "{iothub}/{partition}_{YYYY}_{MM}_{DD}_{HH}_{mm}"
              }
            
              endpoint {
                type              = "AzureIotHub.EventHub"
                connection_string = azurerm_eventhub_authorization_rule.example.primary_connection_string
                name              = "export2"
              }
            
              route {
                name           = "export"
                source         = "DeviceMessages"
                condition      = "true"
                endpoint_names = ["export"]
                enabled        = true
              }
            
              route {
                name           = "export2"
                source         = "DeviceMessages"
                condition      = "true"
                endpoint_names = ["export2"]
                enabled        = true
              }
            
              enrichment {
                key            = "tenant"
                value          = "$twin.tags.Tenant"
                endpoint_names = ["export", "export2"]
              }
              public_network_access_enabled = false
              tags = {
                purpose = "testing"
              }
            }
            """)
        resource_conf = hcl_res['resource'][0]['azurerm_iothub']['example']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)


if __name__ == '__main__':
    unittest.main()



import unittest
import hcl2

from checkov.terraform.checks.resource.aws.SagemakerEndpointConfigurationEncryption import check
from checkov.common.models.enums import CheckResult


class TestSagemakerEndpointConfigurationEncryption(unittest.TestCase):

    def test_failure(self):
        hcl_res = hcl2.loads("""
                resource "aws_sagemaker_endpoint_configuration" "test" {
                    name = "my-endpoint-config"

                    production_variants {
                      variant_name           = "variant-1"
                      model_name             = aws_sagemaker_model.m.name
                      initial_instance_count = 1
                      instance_type          = "ml.t2.medium"
                    }
                }
        """)
        resource_conf = hcl_res['resource'][0]['aws_sagemaker_endpoint_configuration']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.FAILED, scan_result)

    def test_success(self):
        hcl_res = hcl2.loads("""
                resource "aws_sagemaker_endpoint_configuration" "test" {
                    name = "my-endpoint-config"
                    kms_key_arn = aws_kms_key.test.arn
                    
                    production_variants {
                      variant_name           = "variant-1"
                      model_name             = aws_sagemaker_model.m.name
                      initial_instance_count = 1
                      instance_type          = "ml.t2.medium"
                    }
                }
        """)
        resource_conf = hcl_res['resource'][0]['aws_sagemaker_endpoint_configuration']['test']
        scan_result = check.scan_resource_conf(conf=resource_conf)
        self.assertEqual(CheckResult.PASSED, scan_result)

if __name__ == '__main__':
    unittest.main()



def a():
    test_files_dir = current_dir + "/example_LustreFSEncryptedWithCMK"

    mock_url = mock_bc_integration.api_url + "/api/v1/vulnerabilities/scan-results/2e97f5afea42664309f492a1e2083b43479c2936"

    PASSWORD = "PASSWORD"
    STATUS_ERROR_PASSWORD_FETCH = "ERROR_PASSWORD_FETCH"

    return "Properties/LogPublishingOptions/AUDIT_LOGS/Enabled"

    metadata_options['HttpTokens'] == "required"

    "MDEyOk9yZ2FuaXphdGlvbjE\u003d"

    IAM_ROLE_STATEMENTS_TOKEN = 'iamRoleStatements'

    a = {
        'JSON Web Token': 'CKV_SECRET_9',
        'NPM tokens': 'CKV_SECRET_12',
        'Slack Token': 'CKV_SECRET_14',
        'SoftLayer Credentials': 'CKV_SECRET_15',
        'Square OAuth Secret': 'CKV_SECRET_16'
    }

    self.go("GCPMySQLdbInstancePoint_In_TimeRecoveryBackupIsEnabled")

    a = {'s3_origin_config': [{'origin_access_identity': ['origin-access-identity/cloudfront/ABCDEFG1234567']}]}



from __future__ import annotations

from dataclasses import dataclass
from typing import Literal

from checkov.common.graph.graph_builder.graph_components.block_types import BlockType as CommonBlockType


@dataclass
class BlockType(CommonBlockType):
    PARAMETER: Literal["parameters"] = "parameters"
    VARIABLE: Literal["variables"] = "variables"
    OUTPUT: Literal["outputs"] = "outputs"



from aws_cdk import core
from aws_cdk import aws_ec2 as ec2

class MyVpcEndpointServiceStack(core.Stack):
    def __init__(self, scope: core.Construct, id: str, **kwargs) -> None:
        super().__init__(scope, id, **kwargs)

        # Define VPC Endpoint Service with acceptance required
        vpc_endpoint_service = ec2.CfnVPCEndpointService(
            self, 'MyVPCEndpointService',
            acceptance_required=False,
            # Other properties for your VPC Endpoint Service
        )

app = core.App()
MyVpcEndpointServiceStack(app, "MyVpcEndpointServiceStack")
app.synth()
